@@ -1,9 +1,9 @@
 //
-// "$Id: Fl_Function_Type.cxx,v 1.15.2.10 1999/12/17 18:34:55 bill Exp $"
+// "$Id: Fl_Function_Type.cxx,v 1.15.2.16 2001/01/22 15:13:38 easysw Exp $"
 //
 // C function type code for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-1999 by Bill Spitzak and others.
+// Copyright 1998-2001 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -20,7 +20,7 @@
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 // USA.
 //
-// Please report all bugs and problems to "fltk-bugs@easysw.com".
+// Please report all bugs and problems to "fltk-bugs@fltk.org".
 //
 
 #include <FL/Fl.H>
@@ -30,6 +30,13 @@
 #include <stdio.h>
 #include <ctype.h>
 
+extern int i18n_type;
+extern const char* i18n_include;
+extern const char* i18n_function;
+extern const char* i18n_file;
+extern const char* i18n_set;
+extern char i18n_program[];
+
 ////////////////////////////////////////////////////////////////
 // quick check of any C code for legality, returns an error message
 
@@ -66,10 +73,14 @@ const char *_c_check(const char * & c, int type) {
       while ((*c != '*' || c[1] != '/') && *c) c++;
       if (*c == '*') c+=2;
       else {
-        return "unterminated /* ... */ comment";
+        return "missing '*/'";
       }
     }
     break;
+  case '#':
+    // treat cpp directives as a comment:
+    while (*c != '\n' && *c) c++;
+    break;
   case '{':
     if (type==')') goto UNEXPECTED;
     d = _c_check(c,'}');
@@ -89,7 +100,6 @@ const char *_c_check(const char * & c, int type) {
     break;
   case '}':
   case ')':
-  case '#':
   UNEXPECTED:
     if (type == *(c-1)) return 0;
     sprintf(buffer, "unexpected %c", *(c-1));
@@ -213,7 +223,7 @@ void Fl_Function_Type::write_code1() {
       } else rtype = "void";
     }
 
-    const char* k = class_name();
+    const char* k = class_name(0);
     if (k) {
       write_public(public_);
       if (name()[0] == '~')
@@ -465,7 +475,7 @@ void Fl_Decl_Type::write_code1() {
   // lose all trailing semicolons so I can add one:
   const char* e = c+strlen(c);
   while (e>c && e[-1]==';') e--;
-  if (class_name()) {
+  if (class_name(1)) {
     write_public(public_);
     write_h("  %.*s;\n", e-c, c);
   } else {
@@ -551,13 +561,14 @@ void Fl_DeclBlock_Type::write_code2() {
 
 ////////////////////////////////////////////////////////////////
 
-const char* Fl_Type::class_name() const {
+const char* Fl_Type::class_name(const int need_nest) const {
   Fl_Type* p = parent;
   while (p) {
     if (p->is_class()) {
       // see if we are nested in another class, we must fully-qualify name:
       // this is lame but works...
-      const char* q = p->class_name();
+      const char* q = 0;
+      if(need_nest) q=p->class_name(need_nest);
       if (q) {
 	static char buffer[256];
 	if (q != buffer) strcpy(buffer, q);
@@ -661,5 +672,5 @@ void Fl_Class_Type::write_code2() {
 }
 
 //
-// End of "$Id: Fl_Function_Type.cxx,v 1.15.2.10 1999/12/17 18:34:55 bill Exp $".
+// End of "$Id: Fl_Function_Type.cxx,v 1.15.2.16 2001/01/22 15:13:38 easysw Exp $".
 //
