@@ -1,5 +1,5 @@
 //
-// "$Id: Fl_Type.cxx,v 1.6.2.1 1999/03/13 20:07:19 bill Exp $"
+// "$Id: Fl_Type.cxx,v 1.6.2.6 2001/01/22 15:13:38 easysw Exp $"
 //
 // Widget type code for the Fast Light Tool Kit (FLTK).
 //
@@ -15,7 +15,7 @@
 // not in the linked list and are not written to files or
 // copied or otherwise examined.
 //
-// Copyright 1998-1999 by Bill Spitzak and others.
+// Copyright 1998-2001 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -32,7 +32,7 @@
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 // USA.
 //
-// Please report all bugs and problems to "fltk-bugs@easysw.com".
+// Please report all bugs and problems to "fltk-bugs@fltk.org".
 //
 
 #include <FL/Fl.H>
@@ -130,7 +130,8 @@ extern const char* subclassname(Fl_Type*);
 void Widget_Browser::item_draw(void *v, int x, int y, int, int) const {
   Fl_Type *l = (Fl_Type *)v;
   x += 3 + l->level * 10;
-  fl_color(FL_BLACK);
+  if (l->new_selected) fl_color(contrast(FL_BLACK,FL_SELECTION_COLOR));
+  else fl_color(FL_BLACK);
   if (l->is_parent()) {
     if (!l->next || l->next->level <= l->level) {
       if (l->open_!=(l==pushedtitle)) {
@@ -333,6 +334,7 @@ void Fl_Type::remove_child(Fl_Type*) {}
 
 // add a list of widgets as a new child of p:
 void Fl_Type::add(Fl_Type *p) {
+  if (p && parent == p) return;
   parent = p;
   Fl_Type *end = this;
   while (end->next) end = end->next;
@@ -387,6 +389,23 @@ void Fl_Type::insert(Fl_Type *g) {
   widget_browser->redraw();
 }
 
+// Return message number for I18N...
+int
+Fl_Type::msgnum() {
+  int		count;
+  Fl_Type	*p;
+
+  for (count = 0, p = this; p;) {
+    if (p->is_widget() && p->label() && !((Fl_Widget_Type *)p)->image) count ++;
+
+    if (p->prev) p = p->prev;
+    else p = p->parent;
+  }
+
+  return count;
+}
+
+
 // delete from parent:
 Fl_Type *Fl_Type::remove() {
   Fl_Type *end = this;
@@ -648,5 +667,5 @@ void Fl_Type::read_property(const char *c) {
 int Fl_Type::read_fdesign(const char*, const char*) {return 0;}
 
 //
-// End of "$Id: Fl_Type.cxx,v 1.6.2.1 1999/03/13 20:07:19 bill Exp $".
+// End of "$Id: Fl_Type.cxx,v 1.6.2.6 2001/01/22 15:13:38 easysw Exp $".
 //
