@@ -1,10 +1,9 @@
-#include <stdio.h>
 //
-// "$Id: Fl.cxx,v 1.24.2.13 1999/10/23 06:19:59 bill Exp $"
+// "$Id: Fl.cxx,v 1.24.2.41 2001/02/26 00:19:02 spitzak Exp $"
 //
 // Main event handling code for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-1999 by Bill Spitzak and others.
+// Copyright 1998-2001 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -21,7 +20,7 @@
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 // USA.
 //
-// Please report all bugs and problems to "fltk-bugs@easysw.com".
+// Please report all bugs and problems to "fltk-bugs@fltk.org".
 //
 
 #include <FL/Fl.H>
@@ -31,6 +30,7 @@
 #include <stdlib.h>
 #include <string.h>
 
+
 //
 // Globals...
 //
@@ -48,208 +48,242 @@ int		Fl::damage_,
 		Fl::e_clicks,
 		Fl::e_is_click,
 		Fl::e_keysym;
-char		*Fl::e_text = "";
+char		*Fl::e_text = (char *)"";
 int		Fl::e_length;
 
-static double fl_elapsed();
-
 //
 // 'Fl:event_inside()' - Return whether or not the mouse event is inside
 //                       the given rectangle.
 //
 
 int Fl::event_inside(int x,int y,int w,int h) /*const*/ {
-  int mx = event_x() - x;
-  int my = event_y() - y;
+  int mx = e_x - x;
+  int my = e_y - y;
   return (mx >= 0 && mx < w && my >= 0 && my < h);
 }
 
 int Fl::event_inside(const Fl_Widget *o) /*const*/ {
-  return event_inside(o->x(),o->y(),o->w(),o->h());
+  int mx = e_x - o->x();
+  int my = e_y - o->y();
+  return (mx >= 0 && mx < o->w() && my >= 0 && my < o->h());
 }
 
-// Timeouts are insert-sorted into order.  This works good if there
-// are only a small number:
-
-static struct Timeout {
+////////////////////////////////////////////////////////////////
+// Timeouts are stored in a sorted list, so only the first one needs
+// to be checked to see if any should be called.
+  
+struct Timeout {
   double time;
   void (*cb)(void*);
   void* arg;
-} * timeout;
-static int numtimeouts;
-static int timeout_array_size;
-
-void Fl::add_timeout(double t, void (*cb)(void *), void *v) {
-
-  fl_elapsed();
-
-  if (numtimeouts >= timeout_array_size) {
-    timeout_array_size = 2*timeout_array_size+1;
-    timeout = (Timeout*)realloc(timeout, timeout_array_size*sizeof(Timeout));
-  }
-
-  // insert-sort the new timeout:
-  int i;
-  for (i=0; i<numtimeouts; i++) {
-    if (timeout[i].time > t) {
-      for (int j=numtimeouts; j>i; j--) timeout[j] = timeout[j-1];
-      break;
-    }
-  }
-  timeout[i].time = t;
-  timeout[i].cb = cb;
-  timeout[i].arg = v;
-
-  numtimeouts++;
-}
-
-void Fl::remove_timeout(void (*cb)(void *), void *v) {
-  int i,j;
-  for (i=j=0; i<numtimeouts; i++) {
-    if (timeout[i].cb == cb && timeout[i].arg==v) ;
-    else {if (j<i) timeout[j]=timeout[i]; j++;}
-  }
-  numtimeouts = j;
-}
-
-static int call_timeouts() {
-  int expired = 0;
-  while (numtimeouts) {
-    if (timeout[0].time > 0) break;
-    // we must remove timeout from array before doing the callback:
-    void (*cb)(void*) = timeout[0].cb;
-    void *arg = timeout[0].arg;
-    numtimeouts--; expired++;
-    if (numtimeouts) memmove(timeout, timeout+1, numtimeouts*sizeof(Timeout));
-    // now it is safe for the callback to do add_timeout:
-    cb(arg);
-  }
-  return expired;
-}
-
-void Fl::flush() {
-  if (damage()) {
-    damage_ = 0;
-    for (Fl_X* x = Fl_X::first; x; x = x->next) {
-      if (x->w->damage() && x->w->visible_r()) {
-	if (x->wait_for_expose) {
-	  // leave Fl::damage() set so programs can tell damage still exists
-	  damage_ = 1;
-	} else {
-	  x->flush();
-	  x->w->clear_damage();
-	}
-      }
-    }
-  }
-#ifndef WIN32
-  if (fl_display) XFlush(fl_display);
-#endif
-}
-
-extern double fl_wait(int timeout_flag, double timeout);
-extern int fl_ready();
-
-static int initclock; // if false we didn't call fl_elapsed() last time
+  Timeout* next;
+};
+static Timeout* first_timeout, *free_timeout;
 
 #ifndef WIN32
 #include <sys/time.h>
 #endif
 
-// fl_elapsed must return the amount of time since the last time it was
-// called.  To reduce the number of system calls to get the
-// current time, the "initclock" symbol is turned on by an indefinite
-// wait.  This should then reset the measured-from time and return zero
-static double fl_elapsed() {
+// I avoid the overhead of getting the current time when we have no
+// timeouts by setting this flag instead of getting the time.
+// In this case calling elapse_timeouts() does nothing, but records
+// the current time, and the next call will actualy elapse time.
+static char reset_clock = 1;
 
+static void elapse_timeouts() {
 #ifdef WIN32
-
   unsigned long newclock = GetTickCount();
-  const int TICKS_PER_SECOND = 1000; // divisor of the value to get seconds
   static unsigned long prevclock;
-  if (!initclock) {prevclock = newclock; initclock = 1; return 0.0;}
-  else if (newclock < prevclock) return 0.0;
-
-  double t = double(newclock-prevclock)/TICKS_PER_SECOND;
+  double elapsed = (newclock-prevclock)/1000.0;
   prevclock = newclock;
-
 #else
-
   static struct timeval prevclock;
   struct timeval newclock;
   gettimeofday(&newclock, NULL);
-  if (!initclock) {
-    prevclock.tv_sec = newclock.tv_sec;
-    prevclock.tv_usec = newclock.tv_usec;
-    initclock = 1;
-    return 0.0;
-  }
-  double t = newclock.tv_sec - prevclock.tv_sec +
+  double elapsed = newclock.tv_sec - prevclock.tv_sec +
     (newclock.tv_usec - prevclock.tv_usec)/1000000.0;
   prevclock.tv_sec = newclock.tv_sec;
   prevclock.tv_usec = newclock.tv_usec;
-
 #endif
-
-  // expire any timeouts:
-  if (t > 0.0) for (int i=0; i<numtimeouts; i++) timeout[i].time -= t;
-  return t;
+  if (reset_clock) {
+    reset_clock = 0;
+  } else if (elapsed > 0) {
+    for (Timeout* t = first_timeout; t; t = t->next) t->time -= elapsed;
+  }
 }
 
-void (*Fl::idle)();
+// Continuously-adjusted error value, this is a number <= 0 for how late
+// we were at calling the last timeout. This appears to make repeat_timeout
+// very accurate even when processing takes a significant portion of the
+// time interval:
+static double missed_timeout_by;
+
+void Fl::add_timeout(double time, Fl_Timeout_Handler cb, void *arg) {
+  elapse_timeouts();
+  repeat_timeout(time, cb, arg);
+}
+
+void Fl::repeat_timeout(double time, Fl_Timeout_Handler cb, void *arg) {
+  time += missed_timeout_by; if (time < -.05) time = 0;
+  Timeout* t = free_timeout;
+  if (t) free_timeout = t->next;
+  else t = new Timeout;
+  t->time = time;
+  t->cb = cb;
+  t->arg = arg;
+  // insert-sort the new timeout:
+  Timeout** p = &first_timeout; 
+  while (*p && (*p)->time <= time) p = &((*p)->next);
+  t->next = *p;
+  *p = t;
+}
+
+int Fl::has_timeout(Fl_Timeout_Handler cb, void *arg) {
+  for (Timeout* t = first_timeout; t; t = t->next)
+    if (t->cb == cb && t->arg == arg) return 1;
+  return 0;
+}
+
+void Fl::remove_timeout(Fl_Timeout_Handler cb, void *arg) {
+  // This version removes all matching timeouts, not just the first one.
+  // This may change in the future.
+  for (Timeout** p = &first_timeout; *p;) {
+    Timeout* t = *p;
+    if (t->cb == cb && t->arg == arg) {
+      *p = t->next;
+      t->next = free_timeout;
+      free_timeout = t;
+    } else {
+      p = &(t->next);
+    }
+  }
+}
+
+////////////////////////////////////////////////////////////////
+// Checks are just stored in a list. They are called in the reverse
+// order that they were added (this may change in the future).
+// This is a bit messy because I want to allow checks to be added,
+// removed, and have wait() called from inside them, to do this
+// next_check points at the next unprocessed one for the outermost
+// call to Fl::wait().
+
+struct Check {
+  void (*cb)(void*);
+  void* arg;
+  Check* next;
+};
+static Check* first_check, *next_check, *free_check;
+
+void Fl::add_check(Fl_Timeout_Handler cb, void *arg) {
+  Check* t = free_check;
+  if (t) free_check = t->next;
+  else t = new Check;
+  t->cb = cb;
+  t->arg = arg;
+  t->next = first_check;
+  if (next_check == first_check) next_check = t;
+  first_check = t;
+}
+
+void Fl::remove_check(Fl_Timeout_Handler cb, void *arg) {
+  for (Check** p = &first_check; *p;) {
+    Check* t = *p;
+    if (t->cb == cb && t->arg == arg) {
+      if (next_check == t) next_check = t->next;
+      *p = t->next;
+      t->next = free_check;
+      free_check = t;
+    } else {
+      p = &(t->next);
+    }
+  }
+}
+  
+////////////////////////////////////////////////////////////////
+// wait/run/check/ready:
+
+void (*Fl::idle)(); // see Fl_add_idle.cxx for the add/remove functions
+
+extern int fl_wait(double time); // in Fl_x.cxx or Fl_win32.cxx
+
 static char in_idle;
-static void callidle() {
-  if (!Fl::idle || in_idle) return;
-  in_idle = 1;
-  Fl::idle();
-  in_idle = 0;
+
+double Fl::wait(double time_to_wait) {
+  if (first_timeout) {
+    elapse_timeouts();
+    Timeout *t;
+    while ((t = first_timeout)) {
+      if (t->time > 0) break;
+      // The first timeout in the array has expired.
+      missed_timeout_by = t->time;
+      // We must remove timeout from array before doing the callback:
+      void (*cb)(void*) = t->cb;
+      void *arg = t->arg;
+      first_timeout = t->next;
+      t->next = free_timeout;
+      free_timeout = t;
+      // Now it is safe for the callback to do add_timeout:
+      cb(arg);
+    }
+  } else {
+    reset_clock = 1; // we are not going to check the clock
+  }
+  // checks are a bit messy so that add/remove and wait may be called
+  // from inside them without causing an infinite loop:
+  if (next_check == first_check) {
+    while (next_check) {
+      Check* check = next_check;
+      next_check = check->next;
+      (check->cb)(check->arg);
+    }
+    next_check = first_check;
+  }
+  if (idle) {
+    if (!in_idle) {in_idle = 1; idle(); in_idle = 0;}
+    // the idle function may turn off idle, we can then wait:
+    if (idle) time_to_wait = 0.0;
+  }
+  if (first_timeout && first_timeout->time < time_to_wait)
+    time_to_wait = first_timeout->time;
+  if (time_to_wait <= 0.0) {
+    // do flush second so that the results of events are visible:
+    int ret = fl_wait(0.0);
+    flush();
+    return ret;
+  } else {
+    // do flush first so that user sees the display:
+    flush();
+    return fl_wait(time_to_wait);
+  }
+}
+
+#define FOREVER 1e20
+
+int Fl::run() {
+  while (Fl_X::first) wait(FOREVER);
+  return 0;
 }
 
 int Fl::wait() {
-  callidle();
-  int expired = 0;
-  if (numtimeouts) {fl_elapsed(); expired = call_timeouts();}
-  flush();
-  if (!Fl_X::first) return 0; // no windows
-  if ((idle && !in_idle) || expired) {
-    fl_wait(1,0.0);
-  } else if (numtimeouts) {
-    fl_wait(1, timeout[0].time);
-  } else {
-    initclock = 0;
-    fl_wait(0,0);
-  }
-  return 1;
-}
-
-double Fl::wait(double time) {
-  callidle();
-  int expired = 0;
-  if (numtimeouts) {time -= fl_elapsed(); expired = call_timeouts();}
-  flush();
-  double wait_time = (idle && !in_idle) || expired ? 0.0 : time;
-  if (numtimeouts && timeout[0].time < wait_time) wait_time = timeout[0].time;
-  fl_wait(1, wait_time);
-  return time - fl_elapsed();
-}
-
-int Fl::check() {
-  callidle();
-  if (numtimeouts) {fl_elapsed(); call_timeouts();}
-  fl_wait(1, 0.0);
-  flush();
+  wait(FOREVER);
   return Fl_X::first != 0; // return true if there is a window
 }
 
-int Fl::ready() {
-  // if (idle && !in_idle) return 1; // should it do this?
-  if (numtimeouts) {fl_elapsed(); if (timeout[0].time <= 0) return 1;}
-  return fl_ready();
+int Fl::check() {
+  wait(0.0);
+  return Fl_X::first != 0; // return true if there is a window
 }
 
-int Fl::run() {
-  while (wait());
-  return 0;
+extern int fl_ready();
+
+int Fl::ready() {
+  if (first_timeout) {
+    elapse_timeouts();
+    if (first_timeout->time <= 0) return 1;
+  } else {
+    reset_clock = 1;
+  }
+  return fl_ready();
 }
 
 ////////////////////////////////////////////////////////////////
@@ -273,14 +307,43 @@ Fl_Window* fl_find(Window xid) {
   return 0;
 }
 
+Fl_Window* Fl::first_window() {
+  Fl_X* x = Fl_X::first;
+  return x ? x->w : 0;
+}
+
+Fl_Window* Fl::next_window(const Fl_Window* w) {
+  Fl_X* x = Fl_X::i(w)->next;
+  return x ? x->w : 0;
+}
+
+void Fl::first_window(Fl_Window* window) {
+  if (!window || !window->shown()) return;
+  fl_find(fl_xid(window));
+}
+
 void Fl::redraw() {
   for (Fl_X* x = Fl_X::first; x; x = x->next) x->w->redraw();
 }
 
-Fl_Window* Fl::first_window() {Fl_X* x = Fl_X::first; return x ? x->w : 0;}
-
-Fl_Window* Fl::next_window(const Fl_Window* w) {
-  Fl_X* x = Fl_X::i(w)->next; return x ? x->w : 0;}
+void Fl::flush() {
+  if (damage()) {
+    damage_ = 0;
+    for (Fl_X* x = Fl_X::first; x; x = x->next) {
+      if (x->wait_for_expose) {damage_ = 1; continue;}
+      Fl_Window* w = x->w;
+      if (!w->visible_r()) continue;
+      if (w->damage()) {x->flush(); w->clear_damage();}
+      // destroy damage regions for windows that don't use them:
+      if (x->region) {XDestroyRegion(x->region); x->region = 0;}
+    }
+  }
+#ifdef WIN32
+  GdiFlush();
+#else
+  if (fl_display) XFlush(fl_display);
+#endif
+}
 
 ////////////////////////////////////////////////////////////////
 // Event handlers:
@@ -313,6 +376,7 @@ void Fl::focus(Fl_Widget *o) {
   if (grab()) return; // don't do anything while grab is on
   Fl_Widget *p = focus_;
   if (o != p) {
+    Fl::compose_reset();
     focus_ = o;
     fl_oldfocus = 0;
     for (; p && !p->contains(o); p = p->parent()) {
@@ -326,7 +390,6 @@ void Fl::belowmouse(Fl_Widget *o) {
   if (grab()) return; // don't do anything while grab is on
   Fl_Widget *p = belowmouse_;
   if (o != p) {
-    event_is_click(0);
     belowmouse_ = o;
     for (; p && !p->contains(o); p = p->parent()) p->handle(FL_LEAVE);
   }
@@ -355,6 +418,7 @@ void fl_fix_focus() {
   // set focus based on Fl::modal() and fl_xfocus
   Fl_Widget* w = fl_xfocus;
   if (w) {
+    Fl::e_keysym = 0; // make sure widgets don't think a keystroke moved focus
     while (w->parent()) w = w->parent();
     if (Fl::modal()) w = Fl::modal();
     if (!w->contains(Fl::focus()))
@@ -480,12 +544,11 @@ int Fl::handle(int event, Fl_Window* window)
     window = 0;
   case FL_FOCUS:
     fl_xfocus = window;
-    e_keysym = 0; // make sure it is not confused with navigation key
     fl_fix_focus();
     return 1;
 
   case FL_KEYBOARD:
-    fl_xfocus = window; // this should already be set, but just in case.
+    fl_xfocus = window; // this should not happen!  But maybe it does:
 
     // Try it as keystroke, sending it to focus and all parents:
     for (w = grab() ? grab() : focus(); w; w = w->parent())
@@ -496,8 +559,9 @@ int Fl::handle(int event, Fl_Window* window)
 
     // and then try a shortcut with the case of the text swapped, by
     // changing the text and falling through to FL_SHORTCUT case:
-    if (!isalpha(event_text()[0])) return 0;
-    *(char*)(event_text()) ^= ('A'^'a');
+    {char* c = (char*)event_text(); // cast away const
+    if (!isalpha(*c)) return 0;
+    *c = isupper(*c) ? tolower(*c) : toupper(*c);}
     event = FL_SHORTCUT;
 
   case FL_SHORTCUT:
@@ -573,12 +637,20 @@ void Fl_Window::hide() {
 
 #ifdef WIN32
   if (x->private_dc) ReleaseDC(x->xid,x->private_dc);
-  if (x->xid == fl_window) fl_GetDC(0); // releases dc belonging to window
+  if (x->xid == fl_window && fl_gc) {
+    ReleaseDC(fl_window, fl_gc);
+    fl_window = (HWND)-1;
+    fl_gc = 0;
+  }
 #else
   if (x->region) XDestroyRegion(x->region);
 #endif
   XDestroyWindow(fl_display, x->xid);
-
+#ifdef WIN32
+  // Try to stop the annoying "raise another program" behavior
+  if (non_modal() && Fl::first_window() && Fl::first_window()->shown())
+    Fl::first_window()->show();
+#endif
   delete x;
 }
 
@@ -586,18 +658,35 @@ Fl_Window::~Fl_Window() {
   hide();
 }
 
-// Child windows must respond to FL_SHOW and FL_HIDE by actually
-// doing unmap operations.  Outer windows assumme FL_SHOW & FL_HIDE
-// are messages from X:
+// FL_SHOW and FL_HIDE are called whenever the visibility of this widget
+// or any parent changes.  We must correctly map/unmap the system's window.
+
+// For top-level windows it is assummed the window has already been
+// mapped or unmapped!!!  This is because this should only happen when
+// Fl_Window::show() or Fl_Window::hide() is called, or in response to
+// iconize/deiconize events from the system.
 
 int Fl_Window::handle(int event) {
   if (parent()) switch (event) {
   case FL_SHOW:
     if (!shown()) show();
-    else XMapWindow(fl_display, fl_xid(this));
+    else XMapWindow(fl_display, fl_xid(this)); // extra map calls are harmless
     break;
   case FL_HIDE:
-    if (shown()) XUnmapWindow(fl_display, fl_xid(this));
+    if (shown()) {
+      // Find what really turned invisible, if is was a parent window
+      // we do nothing.  We need to avoid unnecessary unmap calls
+      // because they cause the display to blink when the parent is
+      // remapped.  However if this or any intermediate non-window
+      // widget has really had hide() called directly on it, we must
+      // unmap because when the parent window is remapped we don't
+      // want to reappear.
+      if (visible()) {
+       Fl_Widget* p = parent(); for (;p->visible();p = p->parent()) {}
+       if (p->type() >= FL_WINDOW) break; // don't do the unmap
+      }
+      XUnmapWindow(fl_display, fl_xid(this));
+    }
     break;
   }
   return Fl_Group::handle(event);
@@ -690,15 +779,6 @@ void Fl_Window::flush() {
   draw();
 }
 
-int fl_old_shortcut(const char* s) {
-  if (!s || !*s) return 0;
-  int n = 0;
-  if (*s == '#') {n |= FL_ALT; s++;}
-  if (*s == '+') {n |= FL_SHIFT; s++;}
-  if (*s == '^') {n |= FL_CTRL; s++;}
-  return n | *s;
-}
-
 //
-// End of "$Id: Fl.cxx,v 1.24.2.13 1999/10/23 06:19:59 bill Exp $".
+// End of "$Id: Fl.cxx,v 1.24.2.41 2001/02/26 00:19:02 spitzak Exp $".
 //
