@@ -1,9 +1,9 @@
 //
-// "$Id: Fl_Browser_.cxx,v 1.10.2.6 1999/11/16 14:44:43 mike Exp $"
+// "$Id: Fl_Browser_.cxx,v 1.10.2.15 2001/01/22 15:13:39 easysw Exp $"
 //
 // Base Browser widget class for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-1999 by Bill Spitzak and others.
+// Copyright 1998-2001 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -20,7 +20,7 @@
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 // USA.
 //
-// Please report all bugs and problems to "fltk-bugs@easysw.com".
+// Please report all bugs and problems to "fltk-bugs@fltk.org".
 //
 
 #include <FL/Fl.H>
@@ -62,7 +62,7 @@ static void hscrollbar_callback(Fl_Widget* s, void*) {
   ((Fl_Browser_*)(s->parent()))->hposition(int(((Fl_Scrollbar*)s)->value()));
 }
 
-int Fl_Browser_::scrollbar_width_ = 17;
+int Fl_Browser_::scrollbar_width_ = 16;
 
 // return where to draw the actual box:
 void Fl_Browser_::bbox(int& X, int& Y, int& W, int& H) const {
@@ -88,11 +88,19 @@ int Fl_Browser_::leftedge() const {
   return X;
 }
 
-// the scrollbars are resized & placed by draw(), since each one's size
+// The scrollbars may be moved again by draw(), since each one's size
 // depends on whether the other is visible or not.  This skips over
 // Fl_Group::resize since it moves the scrollbars uselessly.
 void Fl_Browser_::resize(int X, int Y, int W, int H) {
   Fl_Widget::resize(X, Y, W, H);
+  // move the scrollbars so they can respond to events:
+  bbox(X,Y,W,H);
+  scrollbar.resize(
+	scrollbar.align()&FL_ALIGN_LEFT ? X-scrollbar_width_ : X+W,
+	Y, scrollbar_width_, H);
+  hscrollbar.resize(
+	X, scrollbar.align()&FL_ALIGN_TOP ? Y-scrollbar_width_ : Y+H,
+	W, scrollbar_width_);
 }
 
 // Cause minimal update to redraw the given item:
@@ -303,14 +311,14 @@ J1:
 	fl_color(active_r() ? selection_color() : inactive(selection_color()));
 	fl_rectf(X, yy+Y, W, hh);
       } else if (!(damage()&FL_DAMAGE_ALL)) {
-	fl_color(active_r() ? color() : inactive(color()));
+	fl_color(color());
 	fl_rectf(X, yy+Y, W, hh);
       }
-      if (type() == FL_MULTI_BROWSER && l == selection_) {
+      item_draw(l, X-hposition_, yy+Y, W+hposition_, hh);
+      if (l == selection_) {
 	fl_color(active_r() ? textcolor() : inactive(textcolor()));
 	fl_rect(X+1, yy+Y, W-2, hh);
       }
-      item_draw(l, X-hposition_, yy+Y, W+hposition_, hh);
       int w = item_width(l);
       if (w > max_width) {max_width = w; max_width_item = l;}
     }
@@ -318,7 +326,7 @@ J1:
   }
   // erase the area below last line:
   if (!(damage()&FL_DAMAGE_ALL) && yy < H) {
-    fl_color(active_r() ? color() : inactive(color()));
+    fl_color(color());
     fl_rectf(X, yy+Y, W, H-yy);
   }
   fl_pop_clip();
@@ -389,14 +397,20 @@ void Fl_Browser_::new_list() {
 // Tell it that this item is going away, and that this must remove
 // all pointers to it:
 void Fl_Browser_::deleting(void* l) {
-  if (displayed(l)) redraw_lines();
-  if (l == selection_) selection_ = 0;
-  if (l == top_) {
-    real_position_ -= offset_;
-    offset_ = 0;
-    top_ = item_next(l);
-    if (!top_) top_ = item_prev(l);
+  if (displayed(l)) {
+    redraw_lines();
+    if (l == top_) {
+      real_position_ -= offset_;
+      offset_ = 0;
+      top_ = item_next(l);
+      if (!top_) top_ = item_prev(l);
+    }
+  } else {
+    // we don't know where this item is, recalculate top...
+    real_position_ = 0;
+    top_ = 0;
   }
+  if (l == selection_) selection_ = 0;
   if (l == max_width_item) {max_width_item = 0; max_width = 0;}
 }
 
@@ -539,24 +553,48 @@ int Fl_Browser_::handle(int event) {
   switch (event) {
   case FL_PUSH:
     if (!Fl::event_inside(X, Y, W, H)) return 0;
-    if (type() == FL_SELECT_BROWSER) deselect();
     my = py = Fl::event_y();
     change = 0;
     if (type() == FL_NORMAL_BROWSER || !top_)
       ;
-    else if (type() == FL_MULTI_BROWSER) {
+    else if (type() != FL_MULTI_BROWSER) {
+      change = select_only(find_item(my), when() & FL_WHEN_CHANGED);
+    } else {
       void* l = find_item(my);
       whichway = 1;
-      if (Fl::event_state(FL_SHIFT|FL_CTRL)) { // toggle selection:
+      if (Fl::event_state(FL_CTRL)) { // toggle selection:
 	if (l) {
 	  whichway = !item_selected(l);
 	  change = select(l, whichway, when() & FL_WHEN_CHANGED);
 	}
-      } else {
+      } else if (Fl::event_state(FL_SHIFT)) { // extend selection:
+	change = 0;
+	if (selection_ && l != selection_) {
+	  void *m = l;
+	  int down = 0;
+	  whichway = item_selected(selection_);
+	  for (m = selection_; m; m = item_next(m)) {
+	    if (m == l) down = 1;
+	  }
+	  void *n;
+	  if (down) {
+	    m = selection_;
+	    n = l;
+	  } else {
+	    m = l;
+	    n = selection_;
+	  }
+	  while (m != n) {
+	    select(m, whichway, when() & FL_WHEN_CHANGED);
+	    m = item_next(m);
+	  }
+	  select(n, whichway, when() & FL_WHEN_CHANGED);
+	  select(l, whichway, when() & FL_WHEN_CHANGED);
+	  change = 1;
+	}
+      } else { // select only this item
 	change = select_only(l, when() & FL_WHEN_CHANGED);
       }
-    } else {
-      change = select_only(find_item(my), when() & FL_WHEN_CHANGED);
     }
     return 1;
   case FL_DRAG:
@@ -668,5 +706,5 @@ void Fl_Browser_::item_select(void*, int) {}
 int Fl_Browser_::item_selected(void* l) const {return l==selection_;}
 
 //
-// End of "$Id: Fl_Browser_.cxx,v 1.10.2.6 1999/11/16 14:44:43 mike Exp $".
+// End of "$Id: Fl_Browser_.cxx,v 1.10.2.15 2001/01/22 15:13:39 easysw Exp $".
 //
