@@ -1,9 +1,9 @@
 //
-// "$Id: Fl_Input_.cxx,v 1.21.2.2 1999/10/30 20:21:30 bill Exp $"
+// "$Id: Fl_Input_.cxx,v 1.21.2.11 2001/01/22 15:13:39 easysw Exp $"
 //
 // Common input widget routines for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-1999 by Bill Spitzak and others.
+// Copyright 1998-2001 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -20,7 +20,7 @@
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 // USA.
 //
-// Please report all bugs and problems to "fltk-bugs@easysw.com".
+// Please report all bugs and problems to "fltk-bugs@fltk.org".
 //
 
 // This is the base class for Fl_Input.  You can use it directly
@@ -40,10 +40,8 @@
 
 ////////////////////////////////////////////////////////////////
 
-// Wordwrap kludge.  It "sort of" works but there are lots of bugs.
-// Really the whole multi-line-input should be moved to a seperate
-// (and much more complex) subclass.  Single-line and multi-line
-// input are really a *lot* different!
+// Wordwrap kludge.  This works ok but is commented out because
+// we want to move multi-line functionality to it's own subclass.
 // To try the wordwrap define this:
 //#define WORDWRAP
 #ifdef WORDWRAP
@@ -60,17 +58,19 @@ const char* Fl_Input_::expand(const char* p, char* buf) const {
   const char* lastspace = p;
   char* lastspace_out = o;
   int width_to_lastspace = 0;
+  int word_count = 0;
 #endif
   if (type()==FL_SECRET_INPUT) {
     while (o<e && p < value_+size_) {*o++ = '*'; p++;}
   } else while (o<e) {
 #ifdef WORDWRAP
-    if (wordwrap && (p >= value_+size_ || *p==' ' || *p=='\n')) {
-      width_to_lastspace += fl_width(lastspace, p-lastspace);
+    if (wordwrap && (p >= value_+size_ || isspace(*p))) {
+      width_to_lastspace += (int)fl_width(lastspace_out, o-lastspace_out);
       if (p > lastspace+1) {
-	if (width_to_lastspace > wordwrap) {
+	if (word_count && width_to_lastspace > wordwrap) {
 	  p = lastspace; o = lastspace_out; break;
 	}
+	word_count++;
       }
       lastspace = p;
       lastspace_out = o;
@@ -197,25 +197,26 @@ void Fl_Input_::drawtext(int X, int Y, int W, int H) {
       curx = int(expandpos(p, value()+position(), buf, 0)+.5);
       if (Fl::focus()==this && !was_up_down) up_down_pos = curx;
       cury = lines*height;
-      if (Fl::focus()==this) {
-	int fullw = int(expandpos(p, e, buf, 0));
-	if (curx > xscroll_+W-20) {
-	  xscroll_ = curx+20-W;
-	  if (xscroll_ > fullw-W+2) xscroll_ = fullw-W+2;
-	  mu_p = 0; erase_cursor_only = 0;
-	}
-	if (curx < xscroll_+20 && xscroll_) {
-	  if (fullw > W-2) xscroll_ = curx-20;
-	  else xscroll_ = 0;
-	  mu_p = 0; erase_cursor_only = 0;
-	}
-	if (xscroll_ < 0) xscroll_ = 0;
+      int newscroll = xscroll_;
+      if (curx > newscroll+W-20) {
+	// figure out scrolling so there is space after the cursor:
+	newscroll = curx+20-W;
+	// figure out the furthest left we ever want to scroll:
+	int ex = int(expandpos(p, e, buf, 0))+2-W;
+	// use minimum of both amounts:
+	if (ex < newscroll) newscroll = ex;
+      } else if (curx < newscroll+20) {
+	newscroll = curx-20;
+      }
+      if (newscroll < 0) newscroll = 0;
+      if (newscroll != xscroll_) {
+	xscroll_ = newscroll;
+	mu_p = 0; erase_cursor_only = 0;
       }
     }
     lines++;
     if (e >= value_+size_) break;
-    if (*e == '\n' || *e == ' ') e++;
-    p = e;
+    p = e+1;
   }
 
   // adjust the scrolling:
@@ -294,8 +295,7 @@ void Fl_Input_::drawtext(int X, int Y, int W, int H) {
   CONTINUE:
     ypos += height;
     if (e >= value_+size_) break;
-    if (*e == '\n' || *e == ' ') e++;
-    p = e;
+    p = e+1;
   }
 
   // for minimal update, erase all lines below last one if necessary:
@@ -313,15 +313,59 @@ static int isword(char c) {
   return (c&128 || isalnum(c) || strchr("#%&-/@\\_~", c));
 }
 
-int Fl_Input_::wordboundary(int i) const {
-  if (i<=0 || i>=size()) return 1;
-  return isword(index(i-1)) != isword(index(i));
+int Fl_Input_::word_end(int i) const {
+  if (type() == FL_SECRET_INPUT) return size();
+  //while (i < size() && !isword(index(i))) i++;
+  while (i < size() && isword(index(i))) i++;
+  return i;
 }
 
-int Fl_Input_::lineboundary(int i) const {
-  if (i<=0 || i>=size()) return 1;
+int Fl_Input_::word_start(int i) const {
+  if (type() == FL_SECRET_INPUT) return 0;
+//   if (i >= size() || !isword(index(i)))
+//     while (i > 0 && !isword(index(i-1))) i--;
+  while (i > 0 && isword(index(i-1))) i--;
+  return i;
+}
+
+int Fl_Input_::line_end(int i) const {
+  if (type() != FL_MULTILINE_INPUT) return size();
+#ifdef WORDWRAP
+  // go to the start of the paragraph:
+  int j = i;
+  while (j > 0 && index(j-1) != '\n') j--;
+  // now measure lines until we get past i, end of that line is real eol:
+  wordwrap = w()-Fl::box_dw(box())-6;
+  setfont();
+  for (const char* p=value()+j; ;) {
+    char buf[MAXBUF];
+    p = expand(p, buf);
+    if (p-value() >= i) return p-value();
+    p++;
+  }
+#else
+  while (i < size() && index(i) != '\n') i++;
+  return i;
+#endif
+}
+
+int Fl_Input_::line_start(int i) const {
   if (type() != FL_MULTILINE_INPUT) return 0;
-  return index(i-1) == '\n' || index(i) == '\n';
+  int j = i;
+  while (j > 0 && index(j-1) != '\n') j--;
+#ifdef WORDWRAP
+  // now measure lines until we get past i, start of that line is real eol:
+  wordwrap = w()-Fl::box_dw(box())-6;
+  setfont();
+  for (const char* p=value()+j; ;) {
+    char buf[MAXBUF];
+    const char* e = expand(p, buf);
+    if (e-value() >= i) return p-value();
+    p = e+1;
+  }
+#else
+  return j;
+#endif
 }
 
 void Fl_Input_::handle_mouse(int X, int Y,
@@ -348,9 +392,8 @@ void Fl_Input_::handle_mouse(int X, int Y,
   for (p=value();; ) {
     e = expand(p, buf);
     theline--; if (theline < 0) break;
-    if (*e == '\n' || *e == ' ') e++;
-    p = e;
     if (e >= value_+size_) break;
+    p = e+1;
   }
   const char *l, *r, *t; double f0 = Fl::event_x()-X+xscroll_;
   for (l = p, r = e; l<r; ) {
@@ -373,18 +416,20 @@ void Fl_Input_::handle_mouse(int X, int Y,
 	if (newpos < size()) newpos++;
 	else newmark--;
       }
-      if (Fl::event_clicks()>1 || type()==FL_SECRET_INPUT) {
-	while (!lineboundary(newpos)) newpos++;
-	while (!lineboundary(newmark)) newmark--;
+      if (Fl::event_clicks() > 1) {
+	newpos = line_end(newpos);
+	newmark = line_start(newmark);
       } else {
-	while (!wordboundary(newpos)) newpos++;
-	while (!wordboundary(newmark)) newmark--;
+	newpos = word_end(newpos);
+	newmark = word_start(newmark);
       }
     } else {
-      if (Fl::event_clicks()>1 || type()==FL_SECRET_INPUT) {
-	while (!lineboundary(newpos)) newpos--;
+      if (Fl::event_clicks() > 1) {
+	newpos = line_start(newpos);
+	newmark = line_end(newmark);
       } else {
-	while (!wordboundary(newpos)) newpos--;
+	newpos = word_start(newpos);
+	newmark = word_end(newmark);
       }
     }
     // if the multiple click does not increase the selection, revert
@@ -427,16 +472,25 @@ int Fl_Input_::position(int p, int m) {
 }
 
 int Fl_Input_::up_down_position(int i, int keepmark) {
-  while (i > 0 && index(i-1) != '\n') i--;	// go to start of line
-  double oldwid = 0.0;
+  // unlike before, i must be at the start of the line already!
+
   setfont();
-  while (index(i) && index(i)!='\n') {
-    double tt = oldwid + fl_width(index(i));
-    if ((oldwid+tt)/2 >= up_down_pos) break;
-    oldwid = tt;
-    i++;
+#ifdef WORDWRAP
+  if (type()==FL_MULTILINE_INPUT)
+    wordwrap = w()-Fl::box_dw(box())-6;
+  else wordwrap = 0;
+#endif
+  char buf[MAXBUF];
+  const char* p = value()+i;
+  const char* e = expand(p, buf);
+  const char *l, *r, *t;
+  for (l = p, r = e; l<r; ) {
+    t = l+(r-l+1)/2;
+    int f = (int)expandpos(p, t, buf, 0);
+    if (f <= up_down_pos) l = t; else r = t-1;
   }
-  int j = position(i, keepmark ? mark_ : i);
+  int j = l-value();
+  j = position(j, keepmark ? mark_ : j);
   was_up_down = 1;
   return j;
 }
@@ -532,9 +586,26 @@ int Fl_Input_::replace(int b, int e, const char* text, int ilen) {
     size_ += ilen;
   }
   undowidget = this;
-  mark_ = position_ = undoat = b+ilen;
+  undoat = b+ilen;
+
+#ifdef WORDWRAP
+  // Insertions into the word at the end of the line will cause it to
+  // wrap to the next line, so we must indicate that the changes may start
+  // right after the whitespace before the current word.  This will
+  // result in sub-optimal update when such wrapping does not happen
+  // but it is too hard to figure out for now...
+  if (type() == FL_MULTILINE_INPUT)
+    while (b > 0 && !isspace(index(b))) b--;
+#endif
+
+  // make sure we redraw the old selection or cursor:
+  if (mark_ < b) b = mark_;
+  if (position_ < b) b = position_;
 
   minimal_update(b);
+
+  mark_ = position_ = undoat;
+
   if (when()&FL_WHEN_CHANGED) do_callback(); else set_changed();
   return 1;
 }
@@ -611,6 +682,7 @@ int Fl_Input_::handletext(int event, int X, int Y, int W, int H) {
       if (!(damage()&FL_DAMAGE_EXPOSE)) {minimal_update(position_); erase_cursor_only = 1;}
     } else //if (Fl::selection_owner() != this)
       minimal_update(mark_, position_);
+  case FL_HIDE:
     if (when() & FL_WHEN_RELEASE) maybe_do_callback();
     return 1;
 
@@ -636,7 +708,7 @@ int Fl_Input_::handletext(int event, int X, int Y, int W, int H) {
     // strip trailing control characters and spaces before pasting:
     const char* t = Fl::event_text();
     const char* e = t+Fl::event_length();
-    if (type()!=FL_MULTILINE_INPUT) while (e > t && *(uchar*)(e-1) <= ' ') e--;
+    if (type() != FL_MULTILINE_INPUT) while (e > t && isspace(*(e-1))) e--;
     return replace(position(), mark(), t, e-t);}
 
   default:
@@ -715,7 +787,7 @@ int Fl_Input_::static_value(const char* str, int len) {
     xscroll_ = yscroll_ = 0;
     minimal_update(0);
   }
-  position(size(), 0);
+  position(0, size());
   return 1;
 }
 
@@ -745,5 +817,5 @@ Fl_Input_::~Fl_Input_() {
 }
 
 //
-// End of "$Id: Fl_Input_.cxx,v 1.21.2.2 1999/10/30 20:21:30 bill Exp $".
+// End of "$Id: Fl_Input_.cxx,v 1.21.2.11 2001/01/22 15:13:39 easysw Exp $".
 //
