@@ -1,9 +1,9 @@
 //
-// "$Id: Fl_Menu.cxx,v 1.18.2.2 1999/08/09 06:19:33 bill Exp $"
+// "$Id: Fl_Menu.cxx,v 1.18.2.12 2001/01/22 15:13:40 easysw Exp $"
 //
 // Menu code for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-1999 by Bill Spitzak and others.
+// Copyright 1998-2001 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -20,7 +20,7 @@
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 // USA.
 //
-// Please report all bugs and problems to "fltk-bugs@easysw.com".
+// Please report all bugs and problems to "fltk-bugs@fltk.org".
 //
 
 // Warning: this menu code is quite a mess!
@@ -112,7 +112,7 @@ int Fl_Menu_Item::measure(int* hp, const Fl_Menu_* m) const {
   l.type = labeltype_;
   l.font = labelsize_ ? labelfont_ : uchar(m ? m->textfont() : FL_HELVETICA);
   l.size = labelsize_ ? labelsize_ : m ? m->textsize() : FL_NORMAL_SIZE;
-  l.color = labelcolor_;
+  l.color = FL_BLACK; // this makes no difference?
   fl_draw_shortcut = 1;
   int w = 0; int h = 0; l.measure(w, hp ? *hp : h);
   fl_draw_shortcut = 0;
@@ -127,7 +127,8 @@ void Fl_Menu_Item::draw(int x, int y, int w, int h, const Fl_Menu_* m,
   l.type = labeltype_;
   l.font = labelsize_ ? labelfont_ : uchar(m ? m->textfont() : FL_HELVETICA);
   l.size = labelsize_ ? labelsize_ : m ? m->textsize() : FL_NORMAL_SIZE;
-  l.color = !active() ? inactive((Fl_Color)labelcolor_) : (Fl_Color)labelcolor_;
+  l.color = labelcolor_ ? labelcolor_ : m ? m->textcolor() : int(FL_BLACK);
+  if (!active()) l.color = inactive((Fl_Color)l.color);
   Fl_Color color = m ? m->color() : FL_GRAY;
   if (selected) {
     Fl_Color r = m ? m->selection_color() : FL_SELECTION_COLOR;
@@ -148,7 +149,7 @@ void Fl_Menu_Item::draw(int x, int y, int w, int h, const Fl_Menu_* m,
       x += 3;
       w -= 8;
     } else {
-      fl_draw_box(b, x+1, y-1, w-2, h+2, r);
+      fl_draw_box(b, x+1, y-(LEADING-2)/2, w-2, h+(LEADING-2), r);
     }
   }
 
@@ -199,18 +200,23 @@ menuwindow::menuwindow(const Fl_Menu_Item* m, int X, int Y, int Wp, int Hp,
   clear_border();
   menu = m;
   drawn_selected = -1;
-  box(button && button->box() ? button->box() : FL_UP_BOX);
+  if (button) {
+    box(button->box());
+    if (box() == FL_NO_BOX || box() == FL_FLAT_BOX) box(FL_UP_BOX);
+  } else {
+    box(FL_UP_BOX);
+  }
   color(button ? button->color() : FL_GRAY);
   selected = -1;
-  {int i = 0;
-  if (m) for (const Fl_Menu_Item* m1=m; ; m1 = m1->next(), i++) {
+  {int j = 0;
+  if (m) for (const Fl_Menu_Item* m1=m; ; m1 = m1->next(), j++) {
     if (picked) {
-      if (m1 == picked) {selected = i; picked = 0;}
-      else if (m1 > picked) {selected = i-1; picked = 0; Wp = Hp = 0;}
+      if (m1 == picked) {selected = j; picked = 0;}
+      else if (m1 > picked) {selected = j-1; picked = 0; Wp = Hp = 0;}
     }
     if (!m1->text) break;
   }
-  numitems = i;}
+  numitems = j;}
 
   if (menubar) {
     itemheight = 0;
@@ -244,16 +250,17 @@ menuwindow::menuwindow(const Fl_Menu_Item* m, int X, int Y, int Wp, int Hp,
 
   if (!Wp) {if (X < 0) X = 0; if (X > Fl::w()-W) X= Fl::w()-W;}
   x(X); w(W);
-  h((numitems ? itemheight*numitems-LEADING : 0)+2*BW+5);
+  h((numitems ? itemheight*numitems-LEADING : 0)+2*BW+3);
   if (selected >= 0)
-    Y = Y+(Hp-itemheight)/2-selected*itemheight-BW+1;
+    Y = Y+(Hp-itemheight)/2-selected*itemheight-BW;
   else
     Y = Y+Hp;
-  if (m) y(Y-1); else {y(Y-3); w(1); h(1);}
+  if (m) y(Y); else {y(Y-2); w(1); h(1);}
 
   if (t) {
-    int ht = menubar_title ? button->h()-6 : Htitle+2*BW+3;
-    title = new menutitle(X, Y-ht-3, Wtitle, ht, t);
+    int dy = menubar_title ? Fl::box_dy(button->box())+1 : 2;
+    int ht = menubar_title ? button->h()-dy*2 : Htitle+2*BW+3;
+    title = new menutitle(X, Y-ht-dy, Wtitle, ht, t);
   } else
     title = 0;
 }
@@ -269,8 +276,8 @@ void menuwindow::position(int X, int Y) {
 }
 
 // scroll so item i is visible on screen
-void menuwindow::autoscroll(int i) {
-  int Y = y()+Fl::box_dx(box())+2+i*itemheight;
+void menuwindow::autoscroll(int n) {
+  int Y = y()+Fl::box_dx(box())+2+n*itemheight;
   if (Y <= Fl::y()) Y = Fl::y()-Y+10;
   else {
     Y = Y+itemheight-Fl::h()-Fl::y();
@@ -283,27 +290,29 @@ void menuwindow::autoscroll(int i) {
 
 ////////////////////////////////////////////////////////////////
 
-void menuwindow::drawentry(const Fl_Menu_Item* m, int i, int erase) {
+void menuwindow::drawentry(const Fl_Menu_Item* m, int n, int erase) {
   if (!m) return; // this happens if -1 is selected item and redrawn
 
   int BW = Fl::box_dx(box());
   int x = BW;
   int W = this->w();
   int w = W-2*BW-1;
-  int y = BW+2+i*itemheight;
+  int y = BW+1+n*itemheight;
   int h = itemheight - LEADING;
 
-  if (erase && i != selected) {
+  if (erase && n != selected) {
     fl_color(button ? button->color() : FL_GRAY);
-    fl_rectf(x+1, y-1, w-2, h+2);
+    fl_rectf(x+1, y-(LEADING-2)/2, w-2, h+(LEADING-2));
   }
 
-  m->draw(x, y, w, h, button, i==selected);
+  m->draw(x, y, w, h, button, n==selected);
 
   // the shortcuts and arrows assumme fl_color() was left set by draw():
   if (m->submenu()) {
-    int y1 = y+(h-14)/2;
-    fl_polygon(x+w-13, y1+2, x+w-13, y1+2+10, x+w-3, y1+2+5);
+    int sz = (h-5)&-2;
+    int y1 = y+(h-sz)/2;
+    int x1 = x+w-sz-3;
+    fl_polygon(x1, y1, x1, y1+sz, x1+sz, y1+sz/2);
   } else if (m->shortcut_) {
     Fl_Font f = button ? button->textfont() : FL_HELVETICA;
     fl_font(f, button ? button->textsize() : FL_NORMAL_SIZE);
@@ -312,9 +321,9 @@ void menuwindow::drawentry(const Fl_Menu_Item* m, int i, int erase) {
 
   if (m->flags & FL_MENU_DIVIDER) {
     fl_color(FL_DARK3);
-    fl_xyline(BW-1, y+h+1, W-2*BW+2);
+    fl_xyline(BW-1, y+h+(LEADING-2)/2, W-2*BW+2);
     fl_color(FL_LIGHT3);
-    fl_xyline(BW-1, y+h+2, W-2*BW+2);
+    fl_xyline(BW-1, y+h+((LEADING-2)/2+1), W-2*BW+2);
   }
 
 }
@@ -328,8 +337,8 @@ void menuwindow::draw() {
   if (damage() != FL_DAMAGE_CHILD) {	// complete redraw
     fl_draw_box(box(), 0, 0, w(), h(), color());
     if (menu) {
-      const Fl_Menu_Item* m; int i;
-      for (m=menu, i=0; m->text; i++, m = m->next()) drawentry(m, i, 0);
+      const Fl_Menu_Item* m; int j;
+      for (m=menu, j=0; m->text; j++, m = m->next()) drawentry(m, j, 0);
     }
   } else {
     if (damage() & FL_DAMAGE_CHILD && selected!=drawn_selected) { // change selection
@@ -340,8 +349,8 @@ void menuwindow::draw() {
   drawn_selected = selected;
 }
 
-void menuwindow::set_selected(int i) {
-  if (i != selected) {selected = i; damage(FL_DAMAGE_CHILD);}
+void menuwindow::set_selected(int n) {
+  if (n != selected) {selected = n; damage(FL_DAMAGE_CHILD);}
 }
 
 ////////////////////////////////////////////////////////////////
@@ -352,41 +361,29 @@ int menuwindow::find_selected(int mx, int my) {
   my -= y();
   if (my < 0 || my >= h()) return -1;
   if (!itemheight) { // menubar
-    int x = 3; int i = 0;
+    int x = 3; int n = 0;
     const Fl_Menu_Item* m = menu;
-    for (; ; m = m->next(), i++) {
+    for (; ; m = m->next(), n++) {
       if (!m->text) return -1;
       x += m->measure(0, button) + 16;
       if (x > mx) break;
     }
-    return i;
+    return n;
   }
   if (mx < Fl::box_dx(box()) || mx >= w()) return -1;
-  int i = (my-Fl::box_dx(box())-1)/itemheight;
-  if (i < 0 || i>=numitems) return -1;
-  return i;
+  int n = (my-Fl::box_dx(box())-1)/itemheight;
+  if (n < 0 || n>=numitems) return -1;
+  return n;
 }
 
-// return horizontal position for item i in a menubar:
-int menuwindow::titlex(int i) {
+// return horizontal position for item n in a menubar:
+int menuwindow::titlex(int n) {
   const Fl_Menu_Item* m;
   int x = 3;
-  for (m=menu; i--; m = m->next()) x += m->measure(0, button) + 16;
+  for (m=menu; n--; m = m->next()) x += m->measure(0, button) + 16;
   return x;
 }
 
-// match shortcuts & label shortcuts, don't search submenus:
-// returns menu item and index
-const Fl_Menu_Item* Fl_Menu_Item::find_shortcut(int* ip) const {
-  const Fl_Menu_Item* m1 = this;
-  for (int ii = 0; m1 && m1->text; m1 = m1->next(1), ii++) {
-    if (m1->activevisible() &&
-	(Fl::test_shortcut(m1->shortcut_)
-	 || Fl_Widget::test_shortcut(m1->text))) {if (ip) *ip=ii; return m1;}
-  }
-  return 0;
-}
-
 ////////////////////////////////////////////////////////////////
 // Fl_Menu_Item::popup(...)
 
@@ -500,11 +497,11 @@ int menuwindow::handle(int e) {
     }
     break;
   case FL_SHORTCUT: {
-    for (int menu = p.nummenus; menu--;) {
-      menuwindow &mw = *(p.p[menu]);
+    for (int mymenu = p.nummenus; mymenu--;) {
+      menuwindow &mw = *(p.p[mymenu]);
       int item; const Fl_Menu_Item* m = mw.menu->find_shortcut(&item);
       if (m) {
-	setitem(m, menu, item);
+	setitem(m, mymenu, item);
 	if (!m->submenu()) p.state = DONE_STATE;
 	return 1;
       }
@@ -514,16 +511,16 @@ int menuwindow::handle(int e) {
   case FL_DRAG: {
     int mx = Fl::event_x_root();
     int my = Fl::event_y_root();
-    int item=0; int menu;
-    for (menu = p.nummenus-1; ; menu--) {
-      item = p.p[menu]->find_selected(mx, my);
+    int item=0; int mymenu;
+    for (mymenu = p.nummenus-1; ; mymenu--) {
+      item = p.p[mymenu]->find_selected(mx, my);
       if (item >= 0) break;
-      if (menu <= 0) break;
+      if (mymenu <= 0) break;
     }
-    setitem(menu, item);
+    setitem(mymenu, item);
     if (e == FL_PUSH) {
       if (p.current_item && p.current_item->submenu() // this is a menu title
-	  && item != p.p[menu]->selected // and it is not already on
+	  && item != p.p[mymenu]->selected // and it is not already on
 	  && !p.current_item->callback_) // and it does not have a callback
 	p.state = MENU_PUSH_STATE;
       else
@@ -645,7 +642,7 @@ const Fl_Menu_Item* Fl_Menu_Item::pulldown(
 	initial_item = 0;
       } else {
 	nX = cw.x() + cw.w();
-	nY = cw.y() + 1 + p.item_number * cw.itemheight;
+	nY = cw.y() + p.item_number * cw.itemheight;
 	title = 0;
       }
       if (initial_item) { // bring up submenu containing initial item:
@@ -658,7 +655,7 @@ const Fl_Menu_Item* Fl_Menu_Item::pulldown(
 	  for (int menu = 0; menu <= p.menu_number; menu++) {
 	    menuwindow* t = p.p[menu];
 	    int nx = t->x()+dx; if (nx < 0) {nx = 0; dx = -t->x();}
-	    int ny = t->y()+dy+1; if (ny < 0) {ny = 0; dy = -t->y()-1;}
+	    int ny = t->y()+dy; if (ny < 0) {ny = 0; dy = -t->y();}
 	    t->position(nx, ny);
 	  }
 	  setitem(p.nummenus-1, n->selected);
@@ -708,6 +705,24 @@ Fl_Menu_Item::popup(
   return pulldown(X, Y, 0, 0, picked, button, title ? &dummy : 0);
 }
 
+// Search only the top level menu for a shortcut.  Either &x in the
+// label or the shortcut fields are used:
+const Fl_Menu_Item* Fl_Menu_Item::find_shortcut(int* ip) const {
+  const Fl_Menu_Item* m = this;
+  if (m) for (int ii = 0; m->text; m = m->next(), ii++) {
+    if (m->activevisible()) {
+      if (Fl::test_shortcut(m->shortcut_)
+	 || Fl_Widget::test_shortcut(m->text)) {
+	if (ip) *ip=ii;
+	return m;
+      }
+    }
+  }
+  return 0;
+}
+
+// Recursive search of all submenus for anything with this key as a
+// shortcut.  Only uses the shortcut field, ignores &x in the labels:
 const Fl_Menu_Item* Fl_Menu_Item::test_shortcut() const {
   const Fl_Menu_Item* m = this;
   const Fl_Menu_Item* ret = 0;
@@ -728,5 +743,5 @@ const Fl_Menu_Item* Fl_Menu_Item::test_shortcut() const {
 }
 
 //
-// End of "$Id: Fl_Menu.cxx,v 1.18.2.2 1999/08/09 06:19:33 bill Exp $".
+// End of "$Id: Fl_Menu.cxx,v 1.18.2.12 2001/01/22 15:13:40 easysw Exp $".
 //
