@@ -1,9 +1,9 @@
 //
-// "$Id: Fl_Menu_add.cxx,v 1.9.2.4 1999/07/12 12:22:45 carl Exp $"
+// "$Id: Fl_Menu_add.cxx,v 1.9.2.11 2001/02/12 15:34:37 easysw Exp $"
 //
 // Menu utilities for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-1999 by Bill Spitzak and others.
+// Copyright 1998-2001 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -20,7 +20,7 @@
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 // USA.
 //
-// Please report all bugs and problems to "fltk-bugs@easysw.com".
+// Please report all bugs and problems to "fltk-bugs@fltk.org".
 //
 
 // Methods to alter the menu in an Fl_Menu_ widget.
@@ -78,15 +78,31 @@ static Fl_Menu_Item* insert(
   return array;
 }
 
+// Comparison that does not care about deleted '&' signs:
+static int compare(const char* a, const char* b) {
+  for (;;) {
+    int n = *a-*b;
+    if (n) {
+      if (*a == '&') a++;
+      else if (*b == '&') b++;
+      else return n;
+    } else if (*a) {
+      a++; b++;
+    } else {
+      return 0;
+    }
+  }
+}
+
 // Add an item.  The text is split at '/' characters to automatically
 // produce submenus (actually a totally unnecessary feature as you can
 // now add submenu titles directly by setting SUBMENU in the flags):
 int Fl_Menu_Item::add(
-  const char *text,
+  const char *mytext,
   int shortcut,
   Fl_Callback *cb,	
   void *data,
-  int flags
+  int myflags
 ) {
   Fl_Menu_Item *array = this;
   Fl_Menu_Item *m = this;
@@ -101,17 +117,17 @@ int Fl_Menu_Item::add(
 
     /* fill in the buf with name, changing \x to x: */
     q = buf;
-    for (p=text; *p && *p != '/'; *q++ = *p++) if (*p=='\\') p++;
+    for (p=mytext; *p && *p != '/'; *q++ = *p++) if (*p=='\\') p++;
     *q = 0;
 
     item = buf;
     if (*item == '_') {item++; flags1 = FL_MENU_DIVIDER;}
     if (*p != '/') break; /* not a menu title */
-    text = p+1;	/* point at item title */
+    mytext = p+1;	/* point at item title */
 
     /* find a matching menu title: */
     for (; m->text; m = m->next())
-      if (m->flags&FL_SUBMENU && !strcmp(item, m->text)) break;
+      if (m->flags&FL_SUBMENU && !compare(item, m->text)) break;
 
     if (!m->text) { /* create a new menu */
       int n = m-array;
@@ -127,11 +143,11 @@ int Fl_Menu_Item::add(
 
   /* find a matching menu item: */
   for (; m->text; m = m->next())
-    if (!strcmp(m->text,item)) break;
+    if (!compare(m->text,item)) break;
 
   if (!m->text) {	/* add a new menu item */
     int n = m-array;
-    array = insert(array, size, n, item, flags|flags1);
+    array = insert(array, size, n, item, myflags|flags1);
     size++;
     if (flags & FL_SUBMENU) { // add submenu delimiter
       array = insert(array, size, n+1, 0, 0);
@@ -144,7 +160,7 @@ int Fl_Menu_Item::add(
   m->shortcut_ = shortcut;
   m->callback_ = cb;
   m->user_data_ = data;
-  m->flags = flags|flags1;
+  m->flags = myflags|flags1;
 
   if (array == local_array) local_array_size = size;
   return m-array;
@@ -177,6 +193,7 @@ int Fl_Menu_::add(const char *t, int s, Fl_Callback *c,void *v,int f) {
       } else {
 	local_array_alloc = 15;
 	local_array = menu_ = new Fl_Menu_Item[local_array_alloc];
+        memset(local_array, 0, sizeof(Fl_Menu_Item) * local_array_alloc);
       }
       menu_[0].text = 0;
       local_array_size = 1;
@@ -225,10 +242,17 @@ void Fl_Menu_::remove(int i) {
   int n = size();
   if (i<0 || i>=n) return;
   if (!alloc) copy(menu_);
-  if (alloc > 1) free((void *)menu_[i].text);
-  memmove(&menu_[i],&menu_[i+1],(n-i)*sizeof(Fl_Menu_Item));
+  // find the next item, skipping submenus:
+  Fl_Menu_Item* item = menu_+i;
+  const Fl_Menu_Item* next_item = item->next();
+  // delete the text only if all items were created with add():
+  if (alloc > 1) {
+    for (Fl_Menu_Item* m = item; m < next_item; m++)
+      if (m->text) free((void*)(m->text));
+  }
+  memmove(item, next_item, ((menu_+n+1)-next_item)*sizeof(Fl_Menu_Item));
 }
 
 //
-// End of "$Id: Fl_Menu_add.cxx,v 1.9.2.4 1999/07/12 12:22:45 carl Exp $".
+// End of "$Id: Fl_Menu_add.cxx,v 1.9.2.11 2001/02/12 15:34:37 easysw Exp $".
 //
