@@ -1,9 +1,9 @@
 //
-// "$Id: Fl_add_idle.cxx,v 1.4 1999/01/07 19:17:30 mike Exp $"
+// "$Id: Fl_add_idle.cxx,v 1.4.2.7 2001/04/13 17:22:43 spitzak Exp $"
 //
 // Idle routine support for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-1999 by Bill Spitzak and others.
+// Copyright 1998-2001 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -20,7 +20,7 @@
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 // USA.
 //
-// Please report all bugs and problems to "fltk-bugs@easysw.com".
+// Please report all bugs and problems to "fltk-bugs@fltk.org".
 //
 
 // Allows you to manage an arbitrary set of idle() callbacks.
@@ -55,8 +55,8 @@ void Fl::add_idle(void (*cb)(void*), void* data) {
   p->data = data;
   if (first) {
     last->next = p;
+    last = p;
     p->next = first;
-    first = p;
   } else {
     first = last = p;
     p->next = p;
@@ -64,6 +64,15 @@ void Fl::add_idle(void (*cb)(void*), void* data) {
   }
 }
 
+int Fl::has_idle(void (*cb)(void*), void* data) {
+  idle_cb* p = first;
+  if (!p) return 0;
+  for (;; p = p->next) {
+    if (p->cb == cb && p->data == data) return 1;
+    if (p==last) return 0;
+  }
+}
+
 void Fl::remove_idle(void (*cb)(void*), void* data) {
   idle_cb* p = first;
   if (!p) return;
@@ -85,5 +94,5 @@ void Fl::remove_idle(void (*cb)(void*), void* data) {
 }
 
 //
-// End of "$Id: Fl_add_idle.cxx,v 1.4 1999/01/07 19:17:30 mike Exp $".
+// End of "$Id: Fl_add_idle.cxx,v 1.4.2.7 2001/04/13 17:22:43 spitzak Exp $".
 //
