@@ -1,9 +1,9 @@
 //
-// "$Id: Fl_win32.cxx,v 1.33.2.17 1999/12/15 04:58:27 bill Exp $"
+// "$Id: Fl_win32.cxx,v 1.33.2.37 2001/04/27 15:43:38 easysw Exp $"
 //
 // WIN32-specific code for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-1999 by Bill Spitzak and others.
+// Copyright 1998-2001 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -20,7 +20,7 @@
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 // USA.
 //
-// Please report all bugs and problems to "fltk-bugs@easysw.com".
+// Please report all bugs and problems to "fltk-bugs@fltk.org".
 //
 
 // This file contains win32-specific code for fltk which is always linked
@@ -32,12 +32,20 @@
 #include <FL/win32.H>
 #include <FL/Fl_Window.H>
 #include <string.h>
+#include <stdio.h>
 #include <stdlib.h>
 #include <sys/types.h>
 #include <time.h>
 #include <winsock.h>
 #include <ctype.h>
 
+//
+// USE_ASYNC_SELECT - define it if you have WSAAsyncSelect()...
+//
+
+// MRS - this currently doesn't appear to work anymore; needs to be fixed!
+//#define USE_ASYNC_SELECT
+
 //
 // WM_SYNCPAINT is an "undocumented" message, which is finally defined in
 // VC++ 6.0.
@@ -50,16 +58,28 @@
 #  define WM_MOUSELEAVE 0x02a3
 #endif
 
+//
+// WM_FLSELECT is the user-defined message that we get when one of
+// the sockets has pending data, etc.
+//
+
+#define WM_FLSELECT	(WM_USER+0x0400)
+
 ////////////////////////////////////////////////////////////////
 // interface to poll/select call:
 
 // fd's are only implemented for sockets.  Microsoft Windows does not
 // have a unified IO system, so it doesn't support select() on files,
-// devices, or pipes...  Also, unlike UNIX the Windows select() call
-// doesn't use the nfds parameter, so we don't need to keep track of
-// the maximum FD number...
+// devices, or pipes...
+//
+// Microsoft provides the Berkeley select() call and an asynchronous
+// select function that sends a WIN32 message when the select condition
+// exists...
 
+#ifndef USE_ASYNC_SELECT
 static fd_set fdsets[3];
+#endif // !USE_ASYNC_SELECT
+
 #define POLLIN 1
 #define POLLOUT 4
 #define POLLERR 8
@@ -84,9 +104,18 @@ void Fl::add_fd(int n, int events, void (*cb)(int, void*), void *v) {
   fd[i].events = events;
   fd[i].cb = cb;
   fd[i].arg = v;
+
+#ifdef USE_ASYNC_SELECT
+  int mask = 0;
+  if (events & POLLIN) mask |= FD_READ;
+  if (events & POLLOUT) mask |= FD_WRITE;
+  if (events & POLLERR) mask |= FD_CLOSE;
+  WSAAsyncSelect(n, fl_window, WM_FLSELECT, mask);
+#else
   if (events & POLLIN) FD_SET(n, &fdsets[0]);
   if (events & POLLOUT) FD_SET(n, &fdsets[1]);
   if (events & POLLERR) FD_SET(n, &fdsets[2]);
+#endif // USE_ASYNC_SELECT
 }
 
 void Fl::add_fd(int fd, void (*cb)(int, void*), void* v) {
@@ -108,9 +137,14 @@ void Fl::remove_fd(int n, int events) {
     j++;
   }
   nfds = j;
+
+#ifdef USE_ASYNC_SELECT
+  WSAAsyncSelect(n, 0, 0, 0);
+#else
   if (events & POLLIN) FD_CLR(unsigned(n), &fdsets[0]);
   if (events & POLLOUT) FD_CLR(unsigned(n), &fdsets[1]);
   if (events & POLLERR) FD_CLR(unsigned(n), &fdsets[2]);
+#endif // USE_ASYNC_SELECT
 }
 
 void Fl::remove_fd(int n) {
@@ -119,23 +153,16 @@ void Fl::remove_fd(int n) {
 
 MSG fl_msg;
 
-int fl_ready() {
-  if (PeekMessage(&fl_msg, NULL, 0, 0, PM_NOREMOVE)) return 1;
-
-  timeval t;
-  t.tv_sec = 0;
-  t.tv_usec = 0;
-  fd_set fdt[3];
-  fdt[0] = fdsets[0];
-  fdt[1] = fdsets[1];
-  fdt[2] = fdsets[2];
-  return ::select(0,&fdt[0],&fdt[1],&fdt[2],&t);
-}
-
-double fl_wait(int timeout_flag, double time) {
+// This is never called with time_to_wait < 0.0.
+// It *should* return negative on error, 0 if nothing happens before
+// timeout, and >0 if any callbacks were done.  This version only
+// returns zero if nothing happens during a 0.0 timeout, otherwise
+// it returns 1.
+int fl_wait(double time_to_wait) {
   int have_message = 0;
   int timerid;
 
+#ifndef USE_ASYNC_SELECT
   if (nfds) {
     // For WIN32 we need to poll for socket input FIRST, since
     // the event queue is not something we can select() on...
@@ -158,48 +185,65 @@ double fl_wait(int timeout_flag, double time) {
 	if (FD_ISSET(f,&fdt[2])) revents |= POLLERR;
 	if (fd[i].events & revents) fd[i].cb(f, fd[i].arg);
       }
+      time_to_wait = 0.0; // just peek for any messages
+    } else {
+      // we need to check them periodically, so set a short timeout:
+      if (time_to_wait > .001) time_to_wait = .001;
     }
   }
+#endif // USE_ASYNC_SELECT
 
-  // get the first message by waiting the correct amount of time:
-  if (!timeout_flag) {
-    // If we are monitoring sockets we need to check them periodically,
-    // so set a timer in this case...
-    if (nfds) {
-      // First see if there is a message waiting...
-      have_message = PeekMessage(&fl_msg, NULL, 0, 0, PM_REMOVE);
-      if (!have_message) {
-	// If not then set a 1ms timer...
-	timerid = SetTimer(NULL, 0, 1, NULL);
-	GetMessage(&fl_msg, NULL, 0, 0);
-	KillTimer(NULL, timerid);
-      }
-    } else {
-      // Wait for a message...
-      GetMessage(&fl_msg, NULL, 0, 0);
-    }
-    have_message = 1;
-  } else {
+  if (time_to_wait < 2147483.648) {
     // Perform the requested timeout...
     have_message = PeekMessage(&fl_msg, NULL, 0, 0, PM_REMOVE);
-    if (!have_message && time > 0.0) {
-      int t = (int)(time * 1000.0);
-      if (t <= 0) t = 1;
+    if (!have_message) {
+      int t = (int)(time_to_wait * 1000.0 + .5);
+      if (t <= 0) return 0; // too short to measure
       timerid = SetTimer(NULL, 0, t, NULL);
-      GetMessage(&fl_msg, NULL, 0, 0);
+      have_message = GetMessage(&fl_msg, NULL, 0, 0);
       KillTimer(NULL, timerid);
-      have_message = 1;
     }
+  } else {
+    have_message = GetMessage(&fl_msg, NULL, 0, 0);
   }
 
-  // execute it, them execute any other messages that become ready during it:
+  // Execute the message we got, and all other pending messages:
   while (have_message) {
+#ifdef USE_ASYNC_SELECT
+    if (fl_msg.message == WM_FLSELECT) {
+      // Got notification for socket
+      for (int i = 0; i < nfds; i ++)
+        if (fd[i].fd == (int)fl_msg.wParam) {
+	  (fd[i].cb)(fd[i].fd, fd[i].arg);
+	  break;
+	}
+      // looks like it is best to do the dispatch-message anyway:
+    }
+#endif
     TranslateMessage(&fl_msg);
     DispatchMessage(&fl_msg);
     have_message = PeekMessage(&fl_msg, NULL, 0, 0, PM_REMOVE);
   }
 
-  return time;
+  // This should return 0 if only timer events were handled:
+  return 1;
+}
+
+// fl_ready() is just like fl_wait(0.0) except no callbacks are done:
+int fl_ready() {
+  if (PeekMessage(&fl_msg, NULL, 0, 0, PM_NOREMOVE)) return 1;
+#ifdef USE_ASYNC_SELECT
+  return 0;
+#else
+  timeval t;
+  t.tv_sec = 0;
+  t.tv_usec = 0;
+  fd_set fdt[3];
+  fdt[0] = fdsets[0];
+  fdt[1] = fdsets[1];
+  fdt[2] = fdsets[2];
+  return ::select(0,&fdt[0],&fdt[1],&fdt[2],&t);
+#endif // USE_ASYNC_SELECT
 }
 
 ////////////////////////////////////////////////////////////////
@@ -375,11 +419,16 @@ static Fl_Window* resize_bug_fix;
 
 static LRESULT CALLBACK WndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
 {
-  static char buffer[2];
+  // Matt: When dragging a full window, MSWindows on 'slow'
+  // machines can lose track of the window refresh area. It sends some kind
+  // of panic message to the desktop that in turn sends this message on to
+  // all applications.
   static int cnt=0;
-
-  if(uMsg == WM_SYNCPAINT) {
-    if(cnt) {
+  if (uMsg == WM_SYNCPAINT) {
+    MSG msg;
+    if ( PeekMessage( &msg, hWnd, WM_PAINT, WM_PAINT, false )==0 )
+      InvalidateRect(hWnd,0,FALSE);
+    if (cnt) {
       InvalidateRect(fl_window,0,FALSE);
       cnt = 0;
     } else cnt = 1;
@@ -400,16 +449,13 @@ static LRESULT CALLBACK WndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lPar
 
   case WM_PAINT: {
 
-    // This might be a better alternative, where we fully ignore NT's
-    // "facilities" for painting. MS expects applications to paint according
-    // to a very restrictive paradigm, and this is the way I found of
-    // working around it. In a sense, we are using WM_PAINT simply as an
-    // "exposure alert", like the X event.
-
     Fl_X *i = Fl_X::i(window);
     i->wait_for_expose = 0;
-    // if region == entire window we should delete i->region, else
+    // We need to merge this damage into fltk's damage.  I do this in
+    // reverse, telling Win32 about fltk's damage and then reading back
+    // the new accumulated region.
     if (window->damage()) {
+      // If there is no region the entire window is damaged
       if (i->region) {
 	InvalidateRgn(hWnd,i->region,FALSE);
 	GetUpdateRgn(hWnd,i->region,0);
@@ -419,10 +465,14 @@ static LRESULT CALLBACK WndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lPar
       GetUpdateRgn(hWnd,i->region,0);
     }
     window->clear_damage(window->damage()|FL_DAMAGE_EXPOSE);
+    // These next two statements should not be here, so that all update
+    // is deferred until Fl::flush() is called during idle.  However Win32
+    // apparently is very unhappy if we don't obey it and draw right now.
+    // Very annoying!
     i->flush();
     window->clear_damage();
     // This convinces MSWindows we have painted whatever they wanted
-    // us to paint, and stops it from sending WM_PAINT messages.
+    // us to paint, and stops it from sending WM_PAINT messages:
     ValidateRgn(hWnd,NULL);
     } break;
 
@@ -453,8 +503,30 @@ static LRESULT CALLBACK WndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lPar
     break;
 
   case WM_SHOWWINDOW:
-    if (!window->parent())
+    if (!window->parent()) {
       Fl::handle(wParam ? FL_SHOW : FL_HIDE, window);
+    }
+    break;
+
+  case WM_ACTIVATEAPP:
+    // From eric@vfx.sel.sony.com, we should process WM_ACTIVATEAPP
+    // messages to restore the correct state of the shift/ctrl/alt/lock
+    // keys...  Added control, shift, alt, and meta keys, and changed
+    // to use GetAsyncKeyState and do it when wParam is 1
+    // (that means we have focus...)
+    if (wParam)
+    {
+      ulong state = 0;
+      if (GetAsyncKeyState(VK_CAPITAL)) state |= FL_CAPS_LOCK;
+      if (GetAsyncKeyState(VK_NUMLOCK)) state |= FL_NUM_LOCK;
+      if (GetAsyncKeyState(VK_SCROLL)) state |= FL_SCROLL_LOCK;
+      if (GetAsyncKeyState(VK_CONTROL)&~1) state |= FL_CTRL;
+      if (GetAsyncKeyState(VK_SHIFT)&~1) state |= FL_SHIFT;
+      if (GetAsyncKeyState(VK_MENU)) state |= FL_ALT;
+      if ((GetAsyncKeyState(VK_LWIN)|GetAsyncKeyState(VK_RWIN))&~1) state |= FL_META;
+      Fl::e_state = state;
+      return 0;
+    }
     break;
 
   case WM_KEYDOWN:
@@ -472,26 +544,27 @@ static LRESULT CALLBACK WndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lPar
   case WM_DEADCHAR:
   case WM_SYSDEADCHAR:
   case WM_CHAR:
-  case WM_SYSCHAR:
-    {ulong state = Fl::e_state & 0xff000000; // keep the mouse button state
-     // if GetKeyState is expensive we might want to comment some of these out:
-      if (GetKeyState(VK_SHIFT)&~1) state |= FL_SHIFT;
-      if (GetKeyState(VK_CAPITAL)) state |= FL_CAPS_LOCK;
-      if (GetKeyState(VK_CONTROL)&~1) state |= FL_CTRL;
-      // Alt gets reported for the Alt-GR switch on foreign keyboards.
-      // so we need to check the event as well to get it right:
-      if ((lParam&(1<<29)) //same as GetKeyState(VK_MENU)
+  case WM_SYSCHAR: {
+    ulong state = Fl::e_state & 0xff000000; // keep the mouse button state
+    // if GetKeyState is expensive we might want to comment some of these out:
+    if (GetKeyState(VK_SHIFT)&~1) state |= FL_SHIFT;
+    if (GetKeyState(VK_CAPITAL)) state |= FL_CAPS_LOCK;
+    if (GetKeyState(VK_CONTROL)&~1) state |= FL_CTRL;
+    // Alt gets reported for the Alt-GR switch on foreign keyboards.
+    // so we need to check the event as well to get it right:
+    if ((lParam&(1<<29)) //same as GetKeyState(VK_MENU)
 	&& uMsg != WM_CHAR) state |= FL_ALT;
-      if (GetKeyState(VK_NUMLOCK)) state |= FL_NUM_LOCK;
-      if (GetKeyState(VK_LWIN)&~1 || GetKeyState(VK_RWIN)&~1) {
-	// WIN32 bug?  GetKeyState returns garbage if the user hit the
-	// meta key to pop up start menu.  Sigh.
-	if ((GetAsyncKeyState(VK_LWIN)|GetAsyncKeyState(VK_RWIN))&~1)
-	  state |= FL_META;
-      }
-      if (GetKeyState(VK_SCROLL)) state |= FL_SCROLL_LOCK;
-      Fl::e_state = state;}
+    if (GetKeyState(VK_NUMLOCK)) state |= FL_NUM_LOCK;
+    if ((GetKeyState(VK_LWIN)|GetKeyState(VK_RWIN))&~1) {
+      // WIN32 bug?  GetKeyState returns garbage if the user hit the
+      // meta key to pop up start menu.  Sigh.
+      if ((GetAsyncKeyState(VK_LWIN)|GetAsyncKeyState(VK_RWIN))&~1)
+	state |= FL_META;
+    }
+    if (GetKeyState(VK_SCROLL)) state |= FL_SCROLL_LOCK;
+    Fl::e_state = state;
     if (lParam & (1<<31)) goto DEFAULT; // ignore up events after fixing shift
+    static char buffer[2];
     if (uMsg == WM_CHAR || uMsg == WM_SYSCHAR) {
       buffer[0] = char(wParam);
       Fl::e_length = 1;
@@ -506,7 +579,7 @@ static LRESULT CALLBACK WndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lPar
     // for (int i = lParam&0xff; i--;)
     while (window->parent()) window = window->window();
     if (Fl::handle(FL_KEYBOARD,window)) return 0;
-    break;
+    break;}
 
   case WM_GETMINMAXINFO:
     Fl_X::i(window)->set_minmax((LPMINMAXINFO)lParam);
@@ -695,9 +768,11 @@ Fl_X* Fl_X::make(Fl_Window* w) {
   int wp = w->w();
   int hp = w->h();
 
+  int showit = 1;
+
   if (w->parent()) {
-    style = WS_CHILD;
-    styleEx = WS_EX_WINDOWEDGE | WS_EX_CONTROLPARENT;
+    style |= WS_CHILD;
+    styleEx |= WS_EX_WINDOWEDGE | WS_EX_CONTROLPARENT;
     parent = fl_xid(w->window());
   } else {
     if (!w->size_range_set) {
@@ -723,7 +798,7 @@ Fl_X* Fl_X::make(Fl_Window* w) {
       case 2: style |= WS_THICKFRAME | WS_MAXIMIZEBOX | WS_CAPTION ; break;
     }
     if (by+bt) {
-      if (!w->non_modal()) style |= WS_SYSMENU | WS_MINIMIZEBOX;
+      if (!w->modal()) style |= WS_SYSMENU | WS_MINIMIZEBOX;
       wp += 2*bx;
       hp += 2*by+bt;
     }
@@ -744,6 +819,7 @@ Fl_X* Fl_X::make(Fl_Window* w) {
       Fl_Window* w = Fl_X::first->w;
       while (w->parent()) w = w->window();
       parent = fl_xid(w);
+      if (!w->visible()) showit = 0;
     }
   }
 
@@ -766,14 +842,16 @@ Fl_X* Fl_X::make(Fl_Window* w) {
   Fl_X::first = x;
 
   x->wait_for_expose = 1;
-  w->set_visible();
-  w->handle(FL_SHOW); // get child windows to appear
-  w->redraw(); // force draw to happen
+  if (fl_show_iconic) {showit = 0; fl_show_iconic = 0;}
+  if (showit) {
+    w->set_visible();
+    w->handle(FL_SHOW); // get child windows to appear
+    w->redraw(); // force draw to happen
+  }
   // If we've captured the mouse, we dont want do activate any
   // other windows from the code, or we loose the capture.
-  ShowWindow(x->xid, fl_show_iconic ? SW_SHOWMINNOACTIVE :
-             fl_capture? SW_SHOWNOACTIVATE : SW_SHOWNORMAL);
-  fl_show_iconic = 0;
+  ShowWindow(x->xid, !showit ? SW_SHOWMINNOACTIVE :
+	     fl_capture? SW_SHOWNOACTIVATE : SW_SHOWNORMAL);
   if (w->modal()) {Fl::modal_ = w; fl_fix_focus();}
   return x;
 }
@@ -896,5 +974,5 @@ void Fl_Window::make_current() {
 }
 
 //
-// End of "$Id: Fl_win32.cxx,v 1.33.2.17 1999/12/15 04:58:27 bill Exp $".
+// End of "$Id: Fl_win32.cxx,v 1.33.2.37 2001/04/27 15:43:38 easysw Exp $".
 //
