@@ -1,9 +1,9 @@
 //
-// "$Id: filename_match.cxx,v 1.5 1999/01/07 21:22:28 mike Exp $"
+// "$Id: filename_match.cxx,v 1.5.2.4 2001/01/22 15:13:40 easysw Exp $"
 //
 // Pattern matching routines for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-1999 by Bill Spitzak and others.
+// Copyright 1998-2001 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -20,7 +20,7 @@
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 // USA.
 //
-// Please report all bugs and problems to "fltk-bugs@easysw.com".
+// Please report all bugs and problems to "fltk-bugs@fltk.org".
 //
 
 /* Adapted from Rich Salz. */
@@ -89,7 +89,7 @@ int filename_match(const char *s, const char *p) {
     case 0:	// end of pattern
       return !*s;
 
-#ifdef WIN32
+#if defined(WIN32) && !defined(__CYGWIN__)
     case '\\':	// quote next character
       if (*p) p++;
       if (*s++ != *(p-1)) return 0;
@@ -109,5 +109,5 @@ int filename_match(const char *s, const char *p) {
 }
 
 //
-// End of "$Id: filename_match.cxx,v 1.5 1999/01/07 21:22:28 mike Exp $".
+// End of "$Id: filename_match.cxx,v 1.5.2.4 2001/01/22 15:13:40 easysw Exp $".
 //
