@@ -1,9 +1,9 @@
 //
-// "$Id: fl_color.cxx,v 1.12 1999/01/25 20:43:05 mike Exp $"
+// "$Id: fl_color.cxx,v 1.12.2.5 2001/01/22 15:13:40 easysw Exp $"
 //
 // Color functions for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-1999 by Bill Spitzak and others.
+// Copyright 1998-2001 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -20,7 +20,7 @@
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 // USA.
 //
-// Please report all bugs and problems to "fltk-bugs@easysw.com".
+// Please report all bugs and problems to "fltk-bugs@fltk.org".
 //
 
 // Implementation of fl_color(i), fl_color(r,g,b).
@@ -89,23 +89,41 @@ static void figure_out_visual() {
 
 }
 
+static unsigned fl_cmap[256] = {
+#include "fl_cmap.h" // this is a file produced by "cmap.C":
+};
+
+#if HAVE_OVERLAY
+Fl_XColor fl_xmap[2][256];
+uchar fl_overlay;
+Colormap fl_overlay_colormap;
+XVisualInfo* fl_overlay_visual;
+ulong fl_transparent_pixel;
+#else
+Fl_XColor fl_xmap[1][256];
+#define fl_overlay 0
+#endif
+
 ////////////////////////////////////////////////////////////////
 // Get an rgb color.  This is easy for a truecolor visual.  For
-// colormapped it picks the closest color out of the fltk colormap
-// but be warned that this results in *two* approximations: one
-// to the fltk colormap, and another to whatever colors X allocates.
+// colormapped it picks the closest color out of the cube in the
+// fltk colormap.  However if this color cube entry has been
+// requested before, you will get the earlier requested color, and
+// even this may be approximated if the X colormap was full.
 
 ulong fl_xpixel(uchar r,uchar g,uchar b) {
   if (!beenhere) figure_out_visual();
 #if USE_COLORMAP
   if (!fl_redmask) {
-    Fl_Color i;
-    if (r == g && r == b) { // get it out of gray ramp
-      i = fl_gray_ramp(r*FL_NUM_GRAY/256);
-    } else {		// get it out of color cube:
-      i = fl_color_cube(r*FL_NUM_RED/256,g*FL_NUM_GREEN/256,b*FL_NUM_BLUE/256);
-    }
-    return fl_xpixel(i);
+    // find closest entry in the colormap:
+    Fl_Color i =
+      fl_color_cube(r*FL_NUM_RED/256,g*FL_NUM_GREEN/256,b*FL_NUM_BLUE/256);
+    Fl_XColor &xmap = fl_xmap[fl_overlay][i];
+    if (xmap.mapped) return xmap.pixel;
+    // if not black or white, change the entry to be an exact match:
+    if (i != FL_COLOR_CUBE && i != 0xFF)
+      fl_cmap[i] = (r<<24)|(g<<16)|(b<<8);
+    return fl_xpixel(i); // allocate an X color
   }
 #endif
   return
@@ -125,23 +143,9 @@ void fl_color(uchar r,uchar g,uchar b) {
 // an X color, and does a least-squares match to find the closest
 // color if X cannot allocate that color.
 
-static unsigned fl_cmap[256] = {
-#include "fl_cmap.h" // this is a file produced by "cmap.C":
-};
-
-#if HAVE_OVERLAY
-Fl_XColor fl_xmap[2][256];
-uchar fl_overlay;
-Colormap fl_overlay_colormap;
-XVisualInfo* fl_overlay_visual;
-ulong fl_transparent_pixel;
-#else
-Fl_XColor fl_xmap[1][256];
-#endif
-
 // calculate what color is actually on the screen for a mask:
 static inline uchar realcolor(uchar color, uchar mask) {
-#if 1
+#if 0
   // accurate version if the display has linear gamma, but fl_draw_image
   // works better with the simpler version on most screens...
   uchar m = mask;
@@ -159,107 +163,107 @@ static inline uchar realcolor(uchar color, uchar mask) {
 }
 
 ulong fl_xpixel(Fl_Color i) {
-
-#if HAVE_OVERLAY
   Fl_XColor &xmap = fl_xmap[fl_overlay][i];
-#else
-  Fl_XColor &xmap = fl_xmap[0][i];
-#endif
   if (xmap.mapped) return xmap.pixel;
 
   if (!beenhere) figure_out_visual();
+
   uchar r,g,b;
   {unsigned c = fl_cmap[i]; r=uchar(c>>24); g=uchar(c>>16); b=uchar(c>>8);}
 
 #if USE_COLORMAP
-  Colormap colormap;
+  Colormap colormap = fl_colormap;
 #if HAVE_OVERLAY
-  if (fl_overlay) {colormap = fl_overlay_colormap; goto J1;}
+  if (fl_overlay) colormap = fl_overlay_colormap; else
 #endif
-  if (!fl_redmask) {
-    colormap = fl_colormap;
-#if HAVE_OVERLAY
-  J1:
-    static XColor* ac[2];
-    XColor*& allcolors = ac[fl_overlay];
-    static int nc[2];
-    int& numcolors = nc[fl_overlay];
-#else
-    static XColor *allcolors;
-    static int numcolors;
+  if (fl_redmask) {
 #endif
-
-    // I don't try to allocate colors with XAllocColor once it fails
-    // with any color.  It is possible that it will work, since a color
-    // may have been freed, but some servers are extremely slow and this
-    // avoids one round trip:
-    if (!numcolors) { // don't try after a failure
-      XColor xcol;
-      xcol.red = r<<8; xcol.green = g<<8; xcol.blue = b<<8;
-      if (XAllocColor(fl_display, colormap, &xcol)) {
-	xmap.mapped = 1;
-	xmap.r = xcol.red>>8;
-	xmap.g = xcol.green>>8;
-	xmap.b = xcol.blue>>8;
-	return xmap.pixel = xcol.pixel;
-      }
-
-      // I only read the colormap once.  Again this is due to the slowness
-      // of round-trips to the X server, even though other programs may alter
-      // the colormap after this and make decisions here wrong.
-#if HAVE_OVERLAY
-      if (fl_overlay) numcolors = fl_overlay_visual->colormap_size; else
-#endif
-	numcolors = fl_visual->colormap_size;
-      if (!allcolors) allcolors = new XColor[numcolors];
-      for (int p = numcolors; p--;) allcolors[p].pixel = p;
-      XQueryColors(fl_display, colormap, allcolors, numcolors);
-    }
-
-    // find least-squares match:
-    int mindist = 0x7FFFFFFF;
-    unsigned int bestmatch = 0;
-    for (unsigned int n = numcolors; n--;) {
-#if HAVE_OVERLAY
-      if (fl_overlay && n == fl_transparent_pixel) continue;
-#endif
-      XColor &a = allcolors[n];
-      int d, t;
-      t = int(r)-int(a.red>>8); d = t*t;
-      t = int(g)-int(a.green>>8); d += t*t;
-      t = int(b)-int(a.blue>>8); d += t*t;
-      if (d <= mindist) {bestmatch = n; mindist = d;}
-    }
-    XColor &p = allcolors[bestmatch];
-
-    // It appears to "work" to not call this XAllocColor, which will
-    // avoid another round-trip to the server.  But then X does not
-    // know that this program "owns" this value, and can (and will)
-    // change it when the program that did allocate it exits:
-    if (XAllocColor(fl_display, colormap, &p)) {
-      xmap.mapped = 1;
-      xmap.pixel = p.pixel;
-    } else {
-      // However, if that XAllocColor fails, I have to give up and
-      // assumme the pixel is ok for the duration of the program.  This
-      // is due to bugs (?) in the Solaris X and some X terminals
-      // where XAllocColor *always* fails when the colormap is full,
-      // even if we ask for a color already in it...
-      xmap.mapped = 2; // 2 prevents XFreeColor from being called
-      xmap.pixel = bestmatch;
-    }
-    xmap.r = p.red>>8;
-    xmap.g = p.green>>8;
-    xmap.b = p.blue>>8;
-    return xmap.pixel;
+    // return color for a truecolor visual:
+    xmap.mapped = 2; // 2 prevents XFreeColor from being called
+    xmap.r = realcolor(r, fl_redmask);
+    xmap.g = realcolor(g, fl_greenmask);
+    xmap.b = realcolor(b, fl_bluemask);
+    return xmap.pixel = 
+      (((r&fl_redmask) << fl_redshift)+
+       ((g&fl_greenmask)<<fl_greenshift)+
+       ((b&fl_bluemask)<< fl_blueshift)
+       ) >> fl_extrashift;
+#if USE_COLORMAP
   }
+#if HAVE_OVERLAY
+  static XColor* ac[2];
+  XColor*& allcolors = ac[fl_overlay];
+  static int nc[2];
+  int& numcolors = nc[fl_overlay];
+#else
+  static XColor *allcolors;
+  static int numcolors;
+#endif
+
+  // I don't try to allocate colors with XAllocColor once it fails
+  // with any color.  It is possible that it will work, since a color
+  // may have been freed, but some servers are extremely slow and this
+  // avoids one round trip:
+  if (!numcolors) { // don't try after a failure
+    XColor xcol;
+    xcol.red = r<<8; xcol.green = g<<8; xcol.blue = b<<8;
+    if (XAllocColor(fl_display, colormap, &xcol)) {
+      xmap.mapped = 1;
+      xmap.r = xcol.red>>8;
+      xmap.g = xcol.green>>8;
+      xmap.b = xcol.blue>>8;
+      return xmap.pixel = xcol.pixel;
+    }
+
+    // I only read the colormap once.  Again this is due to the slowness
+    // of round-trips to the X server, even though other programs may alter
+    // the colormap after this and make decisions here wrong.
+#if HAVE_OVERLAY
+    if (fl_overlay) numcolors = fl_overlay_visual->colormap_size; else
+#endif
+      numcolors = fl_visual->colormap_size;
+    if (!allcolors) allcolors = new XColor[numcolors];
+    for (int p = numcolors; p--;) allcolors[p].pixel = p;
+    XQueryColors(fl_display, colormap, allcolors, numcolors);
+  }
+
+  // find least-squares match:
+  int mindist = 0x7FFFFFFF;
+  unsigned int bestmatch = 0;
+  for (unsigned int n = numcolors; n--;) {
+#if HAVE_OVERLAY
+    if (fl_overlay && n == fl_transparent_pixel) continue;
+#endif
+    XColor &a = allcolors[n];
+    int d, t;
+    t = int(r)-int(a.red>>8); d = t*t;
+    t = int(g)-int(a.green>>8); d += t*t;
+    t = int(b)-int(a.blue>>8); d += t*t;
+    if (d <= mindist) {bestmatch = n; mindist = d;}
+  }
+  XColor &p = allcolors[bestmatch];
+
+  // It appears to "work" to not call this XAllocColor, which will
+  // avoid another round-trip to the server.  But then X does not
+  // know that this program "owns" this value, and can (and will)
+  // change it when the program that did allocate it exits:
+  if (XAllocColor(fl_display, colormap, &p)) {
+    xmap.mapped = 1;
+    xmap.pixel = p.pixel;
+  } else {
+    // However, if that XAllocColor fails, I have to give up and
+    // assumme the pixel is ok for the duration of the program.  This
+    // is due to bugs (?) in the Solaris X and some X terminals
+    // where XAllocColor *always* fails when the colormap is full,
+    // even if we ask for a color already in it...
+    xmap.mapped = 2; // 2 prevents XFreeColor from being called
+    xmap.pixel = bestmatch;
+  }
+  xmap.r = p.red>>8;
+  xmap.g = p.green>>8;
+  xmap.b = p.blue>>8;
+  return xmap.pixel;
 #endif
-  // return color for a truecolor visual:
-  xmap.mapped = 2; // 2 prevents XFreeColor from being called
-  xmap.r = realcolor(r, fl_redmask);
-  xmap.g = realcolor(g, fl_greenmask);
-  xmap.b = realcolor(b, fl_bluemask);
-  return xmap.pixel = fl_xpixel(r,g,b);
 }
 
 Fl_Color fl_color_;
@@ -351,5 +355,5 @@ Fl_Color contrast(Fl_Color fg, Fl_Color bg) {
 }
 
 //
-// End of "$Id: fl_color.cxx,v 1.12 1999/01/25 20:43:05 mike Exp $".
+// End of "$Id: fl_color.cxx,v 1.12.2.5 2001/01/22 15:13:40 easysw Exp $".
 //
