@@ -1,9 +1,9 @@
 //
-// "$Id: fl_file_chooser.cxx,v 1.10 1999/01/07 21:22:28 mike Exp $"
+// "$Id: fl_file_chooser.cxx,v 1.10.2.10 2001/05/05 23:39:01 spitzak Exp $"
 //
 // File chooser widget for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-1999 by Bill Spitzak and others.
+// Copyright 1998-2001 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -20,7 +20,7 @@
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 // USA.
 //
-// Please report all bugs and problems to "fltk-bugs@easysw.com".
+// Please report all bugs and problems to "fltk-bugs@fltk.org".
 //
 
 #include <config.h>
@@ -147,6 +147,9 @@ void* FCB::item_prev(void* p) const {
   return ((dirent**)p)-1;
 }
 
+#ifdef _MSC_VER
+#pragma optimize("a",off) // without this it does not change *e
+#endif
 static int ido_matching(const dirent* p, const char* e, const char* n) {
   // replace / or 1 at end with 0 and do match, then put back.  yukko
   int save = *e; *(char*)e = 0;
@@ -154,6 +157,9 @@ static int ido_matching(const dirent* p, const char* e, const char* n) {
   *(char*)e = save;
   return(r);
 }
+#ifdef _MSC_VER
+#pragma optimize("",on)
+#endif
 
 int FCB::incr_height() const {return textsize()+2;}
 
@@ -225,7 +231,11 @@ int FCB::get(char* buf) {
       for (dirent** r = q+1; n && r < last; r++) {
 	if (!item_height(*r, 0)) continue;
 	int i;
-	for (i=0; i<n && (*q)->d_name[i]==(*r)->d_name[i]; i++);
+#ifdef WIN32
+	for (i=0; i<n && tolower((*q)->d_name[i])==tolower((*r)->d_name[i]); i++) {}
+#else
+	for (i=0; i<n && (*q)->d_name[i]==(*r)->d_name[i]; i++) {}
+#endif
 	n = i;
       }
     }
@@ -247,12 +257,12 @@ void FCB::set(const char* buf) {
   const char* c = buf;
   for (bufdirend=0; *c;) switch(*c++) {
   case '?': case '[': case '*': case '{': ispattern = 1; goto BREAK;
-#if defined(WIN32) || defined(__EMX__)
+#if defined(WIN32) || defined(__EMX__) && !defined(__CYGWIN__)
   case '\\':
 #endif
   case '/': bufdirend=c-buf; break;
   }
-#if defined(WIN32) || defined(__EMX__)
+#if defined(WIN32) || defined(__EMX__) && !defined(__CYGWIN__)
   if ((!bufdirend) && isalpha(buf[0]) && (buf[1]==':')) bufdirend = 2;
 #endif
  BREAK:
@@ -305,7 +315,7 @@ void FCB::set(const char* buf) {
       any = 1;
       const char* a = (*q)->d_name;
       const char* b = buf+bufdirend;
-#ifdef WIN32
+#if defined(WIN32) && !defined(__CYGWIN__)
       while (*b && tolower(*a)==tolower(*b)) {a++; b++;}
 #else
       while (*b && *a==*b) {a++; b++;}
@@ -327,7 +337,7 @@ void FCB::set(const char* buf) {
       any = 1;
       const char* a = (*q)->d_name;
       const char* b = buf+bufdirend;
-#ifdef WIN32
+#if defined(WIN32) && !defined(__CYGWIN__)
       while (*b && tolower(*a)==tolower(*b)) {a++; b++;}
 #else
       while (*b && *a==*b) {a++; b++;}
@@ -405,7 +415,7 @@ static void tab_cb(Fl_Widget*, void* v) {
   }
 }
 
-#if defined(WIN32) || defined(__EMX__)
+#if defined(WIN32) || defined(__EMX__) && !defined(__CYGWIN__)
 // ':' needs very special handling!
 static inline int isdirsep(char c) {return c=='/' || c=='\\';}
 #else
@@ -425,10 +435,10 @@ static void input_cb(Fl_Widget*, void* v) {
   w->browser.set(buf);
 }
 
-static void up_cb(Fl_Widget*, void* v) { // the .. button
-  FCW* w = (FCW*)v;
+static void up_cb(Fl_Widget*wd, void*) { // the .. button
+  FCW* w = (FCW*)(wd->window());
   char* p;
-  char* newname;
+  const char* newname;
   char buf[FL_PATH_MAX];
   p = w->browser.listed+w->browser.dirend-1; // point right before last '/'
   if (p < w->browser.listed)
@@ -452,18 +462,16 @@ static void up_cb(Fl_Widget*, void* v) { // the .. button
 }
 
 static void dir_cb(Fl_Widget* obj, void* v) { // directory buttons
-  FCW* w = (FCW*)v;
-  const char* p = obj->label(); if (*p=='&') p++;
+  FCW* w = (FCW*)(obj->window());
   char buf[FL_PATH_MAX];
-  char* q; for (q=buf; *p && *p!=' '; *q++ = *p++); *q = 0;
-  filename_expand(buf, buf);
+  filename_expand(buf, (const char*)v);
   w->input.value(buf);
   w->input.position(10000);
   w->browser.set(buf);
 }
 
-static void working_cb(Fl_Widget*, void* v) { // directory buttons
-  FCW*w = (FCW*)v;
+static void working_cb(Fl_Widget* obj, void*) { // directory buttons
+  FCW* w = (FCW*)(obj->window());
   char buf[FL_PATH_MAX];
   filename_absolute(buf, "");
   w->input.value(buf);
@@ -472,15 +480,11 @@ static void working_cb(Fl_Widget*, void* v) { // directory buttons
 }
 
 static void files_cb(Fl_Widget* obj, void* v) { // file pattern buttons
-  FCW* w = (FCW*)v;
+  FCW* w = (FCW*)(obj->window());
   char buf[FL_PATH_MAX];
   strcpy(buf, w->input.value());
   char* q = buf+w->browser.dirend;
-  if (obj != w->normal_button) {	// tack on first word of label
-    const char* p = obj->label(); if (*p=='&') p++;
-    for (; *p && *p!=' '; *q++ = *p++);
-  }
-  *q = 0;
+  if (v) strcpy(q, (char*)v); else *q = 0;
   w->input.value(buf);
   w->input.position(10000, w->browser.dirend);
   w->browser.set(buf);
@@ -488,13 +492,13 @@ static void files_cb(Fl_Widget* obj, void* v) { // file pattern buttons
 
 /*----------------------- The Main Routine ----------------------*/
 #define HEIGHT_BOX	(4*WIDTH_SPC+HEIGHT_BUT+HEIGHT_INPUT+HEIGHT_BROWSER)
-#define HEIGHT_BUT	25
-#define HEIGHT_INPUT	30
+#define HEIGHT_BUT	23
+#define HEIGHT_INPUT	23
 #define HEIGHT_BROWSER	(9*HEIGHT_BUT+2) // must be > buttons*HEIGHT_BUT
 #define WIDTH_BOX	(3*WIDTH_SPC+WIDTH_BUT+WIDTH_BROWSER)
 #define WIDTH_BROWSER	350
 #define WIDTH_BUT	125
-#define WIDTH_OK	70
+#define WIDTH_OK	60
 #define WIDTH_SPC	5
 
 int FCW::handle(int event) {
@@ -507,7 +511,9 @@ int FCW::handle(int event) {
 }
 
 // set this to make extra directory-jumping button:
+static Fl_Button* extra_button;
 const char* fl_file_chooser_button;
+const char* fl_file_chooser_data;
 extern const char* fl_ok;
 extern const char* fl_cancel;
 
@@ -534,45 +540,45 @@ FCW::FCW() : Fl_Window(WIDTH_BOX, HEIGHT_BOX),
   cancel_button->shortcut("^[");
 
   obj=new Fl_Button(WIDTH_SPC,but_y,WIDTH_BUT,HEIGHT_BUT, "&Up one directory");
-  obj->callback(up_cb, this);
+  obj->callback(up_cb);
   but_y += HEIGHT_BUT;
 
-  obj = new Fl_Button(WIDTH_SPC, but_y, WIDTH_BUT, HEIGHT_BUT, "&~/ Home");
-  obj->callback(dir_cb, this);
+  obj = new Fl_Button(WIDTH_SPC, but_y, WIDTH_BUT, HEIGHT_BUT, "&~ Home");
+  obj->callback(dir_cb, (void*)"~/");
   but_y += HEIGHT_BUT;
 
   obj = new Fl_Button(WIDTH_SPC, but_y, WIDTH_BUT, HEIGHT_BUT, "&/ Root");
-  obj->callback(dir_cb, this);
+  obj->callback(dir_cb, (void*)"/");
   but_y += HEIGHT_BUT;
 
   obj=new Fl_Button(WIDTH_SPC, but_y, WIDTH_BUT, HEIGHT_BUT, "&Current dir");
-  obj->callback(working_cb, this);
+  obj->callback(working_cb);
   but_y += HEIGHT_BUT;
 
   if (fl_file_chooser_button) {
-    obj=new Fl_Button(WIDTH_SPC,but_y,WIDTH_BUT,HEIGHT_BUT,fl_file_chooser_button);
-    obj->callback(dir_cb, this);
+    obj = extra_button = new Fl_Button(WIDTH_SPC,but_y,WIDTH_BUT,HEIGHT_BUT,fl_file_chooser_button);
+    obj->callback(dir_cb, (void*)fl_file_chooser_button);
     but_y += HEIGHT_BUT;
   }
 
   normal_button = new Fl_Button(WIDTH_SPC, but_y, WIDTH_BUT, HEIGHT_BUT, "");
-  normal_button->callback(files_cb, this);
+  normal_button->callback(files_cb, 0);
   but_y += HEIGHT_BUT;
 
-  obj = new Fl_Button(WIDTH_SPC,but_y, WIDTH_BUT, HEIGHT_BUT, "* &All files");
-  obj->callback(files_cb, this);
+  obj = new Fl_Button(WIDTH_SPC,but_y, WIDTH_BUT, HEIGHT_BUT, "&All files");
+  obj->callback(files_cb, (void*)"*");
   but_y += HEIGHT_BUT;
 
-  obj = new Fl_Button(WIDTH_SPC,but_y,WIDTH_BUT,HEIGHT_BUT, ". &Hidden files");
-  obj->callback(files_cb, this);
+  obj = new Fl_Button(WIDTH_SPC,but_y,WIDTH_BUT,HEIGHT_BUT, "&Hidden files");
+  obj->callback(files_cb, (void*)".");
   but_y += HEIGHT_BUT;
 
-  obj = new Fl_Button(WIDTH_SPC,but_y,WIDTH_BUT,HEIGHT_BUT, "*/ &Directories");
-  obj->callback(files_cb, this);
+  obj = new Fl_Button(WIDTH_SPC,but_y,WIDTH_BUT,HEIGHT_BUT, "&Directories");
+  obj->callback(files_cb, (void*)"*/");
   but_y += HEIGHT_BUT;
 
   resizable(new Fl_Box(browser.x(), but_y,
-		       cancel_button->x()-browser.x(),
+		       ok_button->x()-browser.x(),
 		       browser.y()+browser.h()-but_y));
   // add(input); // put last for better draw() speed
   end();
@@ -584,6 +590,10 @@ char* fl_file_chooser(const char* message, const char* pat, const char* fname)
   static FCW* f; if (!f) f = new FCW();
   f->ok_button->label(fl_ok);
   f->cancel_button->label(fl_cancel);
+  if (extra_button) {
+    extra_button->label(fl_file_chooser_button);
+    extra_button->user_data((void*)(fl_file_chooser_data ? fl_file_chooser_data : fl_file_chooser_button));
+  }
 
   if (pat && !*pat) pat = 0;
   if (fname && *fname) {
@@ -597,6 +607,7 @@ char* fl_file_chooser(const char* message, const char* pat, const char* fname)
   }
   f->browser.pattern = pat;
   f->normal_button->label(pat ? pat : "visible files");
+  f->normal_button->user_data((void*)(pat ? pat : 0));
   f->browser.set(f->input.value());
   f->input.position(10000, f->browser.dirend);
 
@@ -622,5 +633,5 @@ char* fl_file_chooser(const char* message, const char* pat, const char* fname)
 }
 
 //
-// End of "$Id: fl_file_chooser.cxx,v 1.10 1999/01/07 21:22:28 mike Exp $".
+// End of "$Id: fl_file_chooser.cxx,v 1.10.2.10 2001/05/05 23:39:01 spitzak Exp $".
 //
