@@ -1,9 +1,9 @@
 //
-// "$Id: gl_start.cxx,v 1.6.2.1 1999/09/16 05:34:27 bill Exp $"
+// "$Id: gl_start.cxx,v 1.6.2.5 2001/03/14 17:20:02 spitzak Exp $"
 //
 // OpenGL context routines for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-1999 by Bill Spitzak and others.
+// Copyright 1998-2001 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -42,35 +42,27 @@
 #include <FL/Fl_Window.H>
 #include <FL/x.H>
 #include <FL/fl_draw.H>
-
 #include "Fl_Gl_Choice.H"
 
-extern GLXContext fl_first_context; // in Fl_Gl_Choice.C
 extern int fl_clip_state_number; // in fl_rect.C
 
-static GLXContext context;
+static GLContext context;
 static int clip_state_number=-1;
 static int pw, ph;
 
 #ifdef WIN32
-static int default_mode;
+static Fl_Gl_Choice* gl_choice;
 #endif
 
 Region XRectangleRegion(int x, int y, int w, int h); // in fl_rect.C
 
 void gl_start() {
-#ifdef WIN32
-  HDC hdc = fl_private_dc(Fl_Window::current(), default_mode,0);
-#endif
   if (!context) {
 #ifdef WIN32
-    context = wglCreateContext(hdc);
-    if (!fl_first_context) fl_first_context = context;
-    else wglShareLists(fl_first_context, context);
+    if (!gl_choice) Fl::gl_visual(0);
+    context = fl_create_gl_context(Fl_Window::current(), gl_choice);
 #else
-    context = glXCreateContext(fl_display, fl_visual, fl_first_context, 1);
-    if (!context) Fl::fatal("OpenGL does not support this visual");
-    if (!fl_first_context) fl_first_context = context;
+    context = fl_create_gl_context(fl_visual);
 #endif
   }
   fl_set_gl_context(Fl_Window::current(), context);
@@ -100,19 +92,18 @@ void gl_start() {
 }
 
 void gl_finish() {
-#ifdef WIN32
   glFlush();
-#else
+#ifndef WIN32
   glXWaitGL();
 #endif
 }
 
 int Fl::gl_visual(int mode, int *alist) {
-#ifdef WIN32
-  default_mode = mode;
-#else
   Fl_Gl_Choice *c = Fl_Gl_Choice::find(mode,alist);
   if (!c) return 0;
+#ifdef WIN32
+  gl_choice = c;
+#else
   fl_visual = c->vis;
   fl_colormap = c->colormap;
 #endif
@@ -122,5 +113,5 @@ int Fl::gl_visual(int mode, int *alist) {
 #endif
 
 //
-// End of "$Id: gl_start.cxx,v 1.6.2.1 1999/09/16 05:34:27 bill Exp $".
+// End of "$Id: gl_start.cxx,v 1.6.2.5 2001/03/14 17:20:02 spitzak Exp $".
 //
