@@ -1,9 +1,9 @@
 //
-// "$Id: CubeMain.cxx,v 1.2 1999/03/04 13:39:41 mike Exp $"
+// "$Id: CubeMain.cxx,v 1.2.2.5 2001/03/15 22:39:56 easysw Exp $"
 //
 // CubeView class definitions for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-1999 by Bill Spitzak and others.
+// Copyright 1998-2001 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -20,7 +20,7 @@
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 // USA.
 //
-// Please report all bugs and problems to "fltk-bugs@easysw.com".
+// Please report all bugs and problems to "fltk-bugs@fltk.org".
 //
 
 #include <config.h>
@@ -28,7 +28,7 @@
 #include "CubeViewUI.h"
 
 int
-main(int argc, char **argv) {
+main(int /*argc*/, char ** /*argv*/) {
 
     CubeViewUI *cvui=new CubeViewUI;
     
@@ -43,5 +43,5 @@ main(int argc, char **argv) {
 
 
 //
-// End of "$Id: CubeMain.cxx,v 1.2 1999/03/04 13:39:41 mike Exp $".
+// End of "$Id: CubeMain.cxx,v 1.2.2.5 2001/03/15 22:39:56 easysw Exp $".
 //
