@@ -1,12 +1,12 @@
 //
-// "$Id: checkers.cxx,v 1.9.2.3 1999/08/09 14:38:40 mike Exp $"
+// "$Id: checkers.cxx,v 1.9.2.7 2001/04/13 19:07:40 easysw Exp $"
 //
 // Checkers game for the Fast Light Tool Kit (FLTK).
 //
 // Hours of fun: the FLTK checkers game!
 // Based on a very old algorithim, but it still works!
 //
-// Copyright 1998-1999 by Bill Spitzak and others.
+// Copyright 1998-2001 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -23,7 +23,7 @@
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 // USA.
 //
-// Please report all bugs and problems to "fltk-bugs@easysw.com".
+// Please report all bugs and problems to "fltk-bugs@fltk.org".
 //
 
 const char* copyright = 
@@ -966,8 +966,8 @@ void Board::draw() {
     fl_rectf(x*BOXSIZE,0,BORDER,h());
     fl_rectf(0,x*BOXSIZE,w(),BORDER);
   }
-  for (int i = 5; i < 40; i++) if (i != erase_this) {
-    draw_piece(b[i], squarex(i), squarey(i));
+  for (int j = 5; j < 40; j++) if (j != erase_this) {
+    draw_piece(b[j], squarex(j), squarey(j));
   }
   if (showlegal) {
     fl_color(FL_WHITE);
@@ -1004,13 +1004,13 @@ void Board::draw() {
 }
 
 // drag the piece on square i to dx dy, or undo drag if i is zero:
-void Board::drag_piece(int i, int dx, int dy) {
+void Board::drag_piece(int j, int dx, int dy) {
   dy = (dy&-2) | dx&1; // make halftone shadows line up
-  if (i != erase_this) drop_piece(erase_this); // should not happen
+  if (j != erase_this) drop_piece(erase_this); // should not happen
   if (!erase_this) { // pick up old piece
-    dragx = squarex(i); dragy = squarey(i);
-    erase_this = i;
-    dragging = b[i];
+    dragx = squarex(j); dragy = squarey(j);
+    erase_this = j;
+    dragging = b[j];
   }
   if (dx != dragx || dy != dragy) {
     damage(FL_DAMAGE_ALL, dragx, dragy, ISIZE, ISIZE);
@@ -1021,12 +1021,12 @@ void Board::drag_piece(int i, int dx, int dy) {
 }
 
 // drop currently dragged piece on square i
-void Board::drop_piece(int i) {
+void Board::drop_piece(int j) {
   if (!erase_this) return; // should not happen!
   erase_this = 0;
   dragging = 0;
-  int x = squarex(i);
-  int y = squarey(i);
+  int x = squarex(j);
+  int y = squarey(j);
   if (x != dragx || y != dragy) {
     damage(4, dragx, dragy, ISIZE, ISIZE);
     damage(4, x, y, ISIZE, ISIZE);
@@ -1045,9 +1045,9 @@ void Board::animate(node* move, int backwards) {
   int x2 = squarex(t);
   int y2 = squarey(t);
   const int STEPS=35;
-  for (int i=0; i<STEPS; i++) {
-    int x = x1+(x2-x1)*i/STEPS;
-    int y = y1+(y2-y1)*i/STEPS;
+  for (int j=0; j<STEPS; j++) {
+    int x = x1+(x2-x1)*j/STEPS;
+    int y = y1+(y2-y1)*j/STEPS;
     drag_piece(move->from,x,y);
     Fl::flush();
   }
@@ -1365,5 +1365,5 @@ int main(int argc, char **argv) {
 }
 
 //
-// End of "$Id: checkers.cxx,v 1.9.2.3 1999/08/09 14:38:40 mike Exp $".
+// End of "$Id: checkers.cxx,v 1.9.2.7 2001/04/13 19:07:40 easysw Exp $".
 //
