@@ -1,9 +1,9 @@
 //
-// "$Id: demo.cxx,v 1.8 1999/02/22 21:09:13 mike Exp $"
+// "$Id: demo.cxx,v 1.8.2.5 2001/01/22 15:13:41 easysw Exp $"
 //
 // Main demo program for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-1999 by Bill Spitzak and others.
+// Copyright 1998-2001 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -20,13 +20,13 @@
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 // USA.
 //
-// Please report all bugs and problems to "fltk-bugs@easysw.com".
+// Please report all bugs and problems to "fltk-bugs@fltk.org".
 //
 
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
-#if defined(WIN32) && !defined(CYGNUS)
+#if defined(WIN32) && !defined(__CYGWIN__)
 #  include <direct.h>
 #else
 #  include <unistd.h>
@@ -91,7 +91,7 @@ typedef struct {
 MENU menus[MAXMENU];
 int mennumb = 0;
 
-int find_menu(char nnn[])
+int find_menu(const char* nnn)
 /* Returns the number of a given menu name. */
 {
   int i;
@@ -100,7 +100,7 @@ int find_menu(char nnn[])
   return -1;
 }
 
-void create_menu(char nnn[])
+void create_menu(const char* nnn)
 /* Creates a new menu with name nnn */
 {
   if (mennumb == MAXMENU -1) return;
@@ -109,7 +109,7 @@ void create_menu(char nnn[])
   mennumb++;
 }
 
-void addto_menu(char men[], char item[], char comm[])
+void addto_menu(const char* men, const char* item, const char* comm)
 /* Adds an item to a menu */
 {
   int n = find_menu(men);
@@ -160,7 +160,7 @@ int numb2but(int inumb, int maxnumb)
 char stack[64][32];
 char stsize = 0;
 
-void push_menu(char nnn[])
+void push_menu(const char* nnn)
 /* Pushes a menu to be visible */
 {
   int n,i,bn;
@@ -267,7 +267,7 @@ void doback(Fl_Widget *, void *) {pop_menu();}
 
 void doexit(Fl_Widget *, void *) {exit(0);}
 
-int load_the_menu(const char fname[])
+int load_the_menu(const char* fname)
 /* Loads the menu file. Returns whether successful. */
 {
   FILE *fin;
@@ -320,6 +320,7 @@ int main(int argc, char **argv) {
   if (!Fl::args(argc,argv,i) || i < argc-1)
     Fl::fatal("Usage: %s <switches> <menufile>\n%s",Fl::help);
   if (i < argc) fname = argv[i];
+
   if (!load_the_menu(fname)) Fl::fatal("Can't open %s",fname);
   strcpy(buf,fname);
   const char *c = filename_name(buf);
@@ -331,6 +332,6 @@ int main(int argc, char **argv) {
 }
 
 //
-// End of "$Id: demo.cxx,v 1.8 1999/02/22 21:09:13 mike Exp $".
+// End of "$Id: demo.cxx,v 1.8.2.5 2001/01/22 15:13:41 easysw Exp $".
 //
 
